
# basically remap the eve "Polygon" command to work specially
# for the kind of input we know we'll be getting.  We know the
# input will be the 3 vertices of a triangle, and we'll need to
# calculate the normal

proc Polygon {P pointsList} {  upvar fileId fileId;

  set x0 [lindex $pointsList 0]
  set y0 [lindex $pointsList 1]
  set z0 [lindex $pointsList 2]
  set x1 [lindex $pointsList 3]
  set y1 [lindex $pointsList 4]
  set z1 [lindex $pointsList 5]
  set x2 [lindex $pointsList 6]
  set y2 [lindex $pointsList 7]
  set z2 [lindex $pointsList 8]


  set aX [expr {$x1 - $x0}]
  set aY [expr {$y1 - $y0}]
  set aZ [expr {$z1 - $z0}]

  set bX [expr {$x2 - $x0}]
  set bY [expr {$y2 - $y0}]
  set bZ [expr {$z2 - $z0}]

  set aMag [expr {sqrt(($aX * $aX) + ($aY * $aY) + ($aZ * $aZ))}]
  set bMag [expr {sqrt(($bX * $bX) + ($bY * $bY) + ($bZ * $bZ))}]

  if {$aMag} \
  {  set uAX [expr {$aX/$aMag}]
     set uAY [expr {$aY/$aMag}]
     set uAZ [expr {$aZ/$aMag}]
  } \
  {  set uAX 0
     set uAY 0
     set uAZ 0
  } 
  if {$bMag} \
  {  set uBX [expr {$bX/$bMag}]
     set uBY [expr {$bY/$bMag}]
     set uBZ [expr {$bZ/$bMag}]
  } \
  {  set uBX 0
     set uBY 0
     set uBZ 0
  } 

  #compute the cross product 
  set nX [expr {($uAY * $uBZ) - ($uAZ * $uBY)}]
  set nY [expr {($uAZ * $uBX) - ($uAX * $uBZ)}]
  set nZ [expr {($uAX * $uBY) - ($uAY * $uBX)}]

  puts $fileId "\tfacet normal $nX $nY $nZ"
  puts $fileId "\t\touter loop"
  puts $fileId "\t\t\tvertex $x0 $y0 $z0"
  puts $fileId "\t\t\tvertex $x1 $y1 $z1"
  puts $fileId "\t\t\tvertex $x2 $y2 $z2"
  puts $fileId "\t\tendloop"
  puts $fileId "\tendfacet"
}

proc eve2stl {polyFile filename} {

  set fileId [open $filename w]

  puts $fileId "solid ascii"
  source $polyFile
  puts $fileId "end solid"

  close $fileId
}
