// copyright 1993 Michael B. Johnson; some portions copyright 1994, MIT
// see COPYRIGHT for reuse legalities
//

#import "EveVarTypeInfo.h"
#import "WWInterp.h"

@implementation EveVarTypeInfo

- init
{
  [super init];

  minExtentP = NO;
  maxExtentP = NO;
  varType = WWTCL_ANY_t;
  varName = NULL;
  varValue = NULL;
  interp = nil;
  enumList = [[List alloc] init];

  return self;
}


- initForVar:(const char *)newVar withInterp:newInterp
{  
  [self init];
  interp = newInterp;
  varName = NXCopyStringBuffer(newVar);

  return self;
}


- free
{
  if (varName) { free(varName); }
  if (varValue) { free(varValue); }

  return [super free];
}


- setValue:(const char *)newValue
{
  if (varValue) { free(varValue); }
  varValue = NXCopyStringBuffer(newValue);
  return self;
}


- (const char *)value { return varValue; }


// what various types will we support?
// - enumerated strings (with regular expression matching)
// - colors (a list of 3 floats, with extents of 0 to 1)
// - floats (with min and max extents)
// - ints (with min and max extents)
//
//
- setTypeEnum {  varType = WWTCL_ENUM_t;  return self;}
- setTypeFloat { varType = WWTCL_FLOAT_t; return self; }
- setTypeInt {  varType = WWTCL_INT_t;  return self; }
- setTypeColor {  varType = WWTCL_COLOR_t;  return self; }


- addEnum:(const char *)newEnum
{
  [enumList addObject:[[WWSimpleString alloc] initWithStr:newEnum]];
  return self;
}


- (BOOL)isThisAValidEnumValue:(const char *)potentialValue
{
  int  i, howMany = [enumList count];


  //if this isn't an enum variable, return NO
  if ((varType != WWTCL_ANY_t) && (varType != WWTCL_ENUM_t))
  {  return NO;
  }

  // for each member of the enum list, check to see if it matches
  for (i = 0; i < howMany; i++)
  {  if ([interp regExpMatch:(char *)potentialValue against:(char *)[[enumList objectAt:i] str]])
     {  return YES;
     }
  }

  return NO;
} 


@end
