// copyright 1993 Michael B. Johnson; some portions copyright 1994, MIT
// see COPYRIGHT for reuse legalities
//

#import "RIBArchiveRecord.h"

@implementation RIBArchiveRecord

+ initialize { return [RIBArchiveRecord setVersion:1], self; }

- init
{
  [super init];
  type = NULL;
  format = NULL;

  return self;
}

- free
{
  if (type)
  {  free(type);
  }
  if (format)
  {  free(format);
  }

  return [super free];
}

// this is used only internally, for setting a copy without freeing the original
- _setType:(const char *)newType format:(const char *)newFormat
{
  if (newType)
  {  type = strcpy((char *)malloc(1 + strlen(newType)), newType); 
  }
  else
  {  type = NULL;
  }
  

  if (newFormat)
  {  format = strcpy((char *)malloc(1 + strlen(newFormat)), newFormat); 
  }
  else
  {  format = NULL;
  }

  return self;
}

- copyFromZone:(NXZone *)zone
{
   id   newCopy = [super copyFromZone:zone];

  [newCopy _setType:type format:format];
  return newCopy;
}

- setType:(const char *)newType format:(const char *)newFormat
{
  if (type)
  {  free(type);
  }
  type = strcpy((char *)malloc(1 + strlen(newType)), newType); 

  if (format)
  {  free(format);
  }
  format = strcpy((char *)malloc(1 + strlen(newFormat)), newFormat); 

  return self;
}

- (char *)type { return type; }
- (char *)format { return format; }

- renderSelf:(WW3DCamera *)camera startingAt:(RtFloat)shutterOpenTime endingAt:(RtFloat)shutterCloseTime
{
  RiArchiveRecord(type, format);

  return self;
}

- (unsigned long int)maxSampleBandwidth
{  
   unsigned long int maxSampleBandwidth = [super maxSampleBandwidth];


   if (type) { maxSampleBandwidth += 1 + strlen(type); }
   if (format) { maxSampleBandwidth += 1 + strlen(format); }

   return maxSampleBandwidth;
}

- writeEve:(NXStream *)stream atTabLevel:(int)tab
{
   int  i;


   for (i = 0; i < tab; i++)
   {  NXPrintf(stream, "\t");
   }
   NXPrintf(stream, "ArchiveRecord {%s} {%s};", type, format); 
   return self;
}

#define typeVector "**"
#define typeValues &type, &format

- read:(NXTypedStream*)stream 
{
    int version;
    [super read:stream];

    version = NXTypedStreamClassVersion(stream,"RIBArchiveRecord");
    if (version == 0) NXReadTypes(stream,"i",&version), version=1;
    if (version == 1)
    {  NXReadTypes(stream,typeVector,typeValues);
    } 
    else 
    {
    }
    return self;
}

- write:(NXTypedStream*)stream 
{
    [super write:stream];

    NXWriteTypes(stream,typeVector, typeValues);

    return self;
}

@end
