// copyright 1993 Michael B. Johnson; some portions copyright 1994, MIT
// see COPYRIGHT for reuse legalities
//


#import "RIBColor.h"

@implementation RIBColor

+ initialize { return [RIBColor setVersion:1], self; }

- init
{
  [super init];
  color[0] = 1.0;
  color[1] = 1.0;
  color[2] = 1.0;

  return self;
}

- setRtColor:(RtColor)newColor
{
  color[0] = newColor[0];
  color[1] = newColor[1];
  color[2] = newColor[2];
  return self;
}

- getRtColor:(RtColor *)colorContainer
{
  (*colorContainer)[0] = color[0];
  (*colorContainer)[1] = color[1];
  (*colorContainer)[2] = color[2];
  return self;
}

- (RtColor *)color {  return &color; }

- (BOOL)theSameAs:otherRIBCommand
{
  RtColor  *otherColorPtr;


  otherColorPtr = [(RIBColor *)otherRIBCommand color];
  if ((*otherColorPtr)[0] != color[0])
  {  return NO;
  }
  if ((*otherColorPtr)[1] != color[1])
  {  return NO;
  }
  if ((*otherColorPtr)[2] != color[2])
  {  return NO;
  }

  return [super theSameAs:otherRIBCommand];
}



- renderSelf:(WW3DCamera *)camera  startingAt:(RtFloat)shutterOpenTime endingAt:(RtFloat)shutterCloseTime
{
  RiColor(color);

  return self;
}

// In prman 3.4, shading parameters don't motion blur...
//- (BOOL)isMotionBlurrable { return YES; }


- (unsigned long int)maxSampleBandwidth {  return ([super maxSampleBandwidth] + (unsigned long int)(3 * sizeof(RtFloat))); }


- (BOOL)isLerpable { return YES; }

// note: because we've made the WWSampleList "safe" for having
// multiple samples with the same data, it's perfectly valid to return
// yourself or b
- lerpWith:b by:(float)uValue
{
   id       newMe = nil;
   RtColor  aColor, bColor, newColor;


   if (([self class] != [b class]) || (uValue <= 0.0))
   {  return self;
   }

   if (uValue >= 1.0)
   {  return b;
   }

   newMe = [super lerpWith:b by:uValue]; // this makes a copy for us

   // okay, now do the specific stuff for this class
   [self getRtColor:&aColor];
   [b getRtColor:&bColor];
   newColor[0] = aColor[0] + ((bColor[0] - aColor[0]) * uValue);
   newColor[1] = aColor[1] + ((bColor[1] - aColor[1]) * uValue);
   newColor[2] = aColor[2] + ((bColor[2] - aColor[2]) * uValue);
   [newMe setRtColor:newColor];

   return newMe;
}


- lerpSelfWith:b by:(float)uValue
{
   id      newMe = nil;
   RtColor  aColor, bColor, newColor;


   if (([self class] != [b class]) || (uValue <= 0.0))
   {  return self;
   }

   if (uValue >= 1.0)
   {  return b;
   }

   [super lerpSelfWith:b by:uValue]; // this makes a copy for us

   // okay, now do the specific stuff for this class
   [self getRtColor:&aColor];
   [b getRtColor:&bColor];
   newColor[0] = aColor[0] + ((bColor[0] - aColor[0]) * uValue);
   newColor[1] = aColor[1] + ((bColor[1] - aColor[1]) * uValue);
   newColor[2] = aColor[2] + ((bColor[2] - aColor[2]) * uValue);
   [newMe setRtColor:newColor];

   return self;
}

- writeEve:(NXStream *)stream atTabLevel:(int)tab
{
   int  i;


   for (i = 0; i < tab; i++)
   {  NXPrintf(stream, "\t");
   }
   NXPrintf(stream, "Color {%f %f %f};", color[0], color[1], color[2]); 
   return self;
}

- writeInventorAtTime:(float)currentTime to:(NXStream *)stream atTabLevel:(int)tab
{
   int  i;


   for (i = 0; i < tab; i++)
   {  NXPrintf(stream, "\t");
   }
   NXPrintf(stream, "BaseColor {\n");
   for (i = 0; i < (tab+1); i++)
   {  NXPrintf(stream, "\t");
   }
   NXPrintf(stream, "rgb %f %f %f\n", color[0], color[1], color[2]);
   for (i = 0; i < tab; i++)
   {  NXPrintf(stream, "\t");
   }
   NXPrintf(stream, "}");

   return self;

}


- read:(NXTypedStream*)stream 
{
    int version;
    [super read:stream];

NX_DURING
    version = NXTypedStreamClassVersion(stream,"RIBColor");
    if (version == 0) NXReadTypes(stream,"i",&version), version=1;
    if (version == 1)
    {  NXReadArray(stream, "f", 3, color);
    } 
NX_HANDLER
   NXLogError("in read: %s, exception [%d] raised.\n", 
                 [[self class] name], NXLocalHandler.code);
   return nil;
NX_ENDHANDLER
    return self;
}

- write:(NXTypedStream*)stream 
{
    [super write:stream];

    NXWriteArray(stream, "f", 3, color);

    return self;
}

@end
