#import <appkit/Control.h>
#import "PAThumbWheelCell.h"

/******************************************************************************
	PAThumbWheelCell
	
PAThumbWheel offers the functionality of Slider plus the features that you would expect from a real thumbwheel (including 2 3/4 D Graphics!).

PAThumbWheel has a linear display mode and a radial display mode and offers the ability to assign a value to the visible region of the control as well as an absolute value that the ThumbWheel will either ignore, bound to or wrap around.

PAThumbWheel can also return relative values via its -relativeIntValue & -relativeFloatValue methods. A snap back option allows mouse loops to start from and return to a base value.

Copyright 1992, Jeff Martin. (jmartin@next.com 415-780-3833)
******************************************************************************/

@interface PAThumbWheel:Control
{
}

+ setCellClass:factoryId;
- initFrame:(const NXRect *)f;

// Direction: vertical or horizontal
- (int)direction;
- setDirection:(int)dir;
- takeDirectionFromMatrix:sender;
- (BOOL)isVertical;
- setVertical;
- (BOOL)isHorizontal;
- setHorizontal;

// DisplayMode: radial or linear
- (int)displayMode;
- setDisplayMode:(int)mode;
- takeDisplayModeFromMatrix:sender;
- toggleDisplayMode:sender;
- (BOOL)isRadial;
- setRadial;
- (BOOL)isLinear;
- setLinear;

// Visible min and max
- (float)visibleMax;
- setVisibleMax:(float)max;
- takeVisibleMaxFrom:sender;
- (float)visibleMin;
- setVisibleMin:(float)min;
- takeVisibleMinFrom:sender;
- (float)visibleRange;
- (float)middleValue;

// Absolute mode: unbounded, bounded or wrapped
- (int)absoluteMode;
- takeAbsoluteModeFromMatrix:sender;
- setAbsoluteMode:(int)mode;
- (BOOL)isUnbounded;
- setUnbounded;
- (BOOL)isBounded;
- setBounded;
- (BOOL)isWrapped;
- setWrapped;
- (float)absoluteMax;
- setAbsoluteMax:(float)value;
- takeAbsoluteMaxFrom:sender;
- (float)absoluteMin;
- setAbsoluteMin:(float)value;
- takeAbsoluteMinFrom:sender;
- (float)absoluteRange;

// Relative Values
- (int)relativeIntValue;
- (float)relativeFloatValue;
- resetRelativeValue;

// Snap back characteristic
- (BOOL)snapsBack;
- setSnapsBack:(BOOL)flag;
- takeSnapsBackFromSwitch:sender;
- (float)snapBackValue;
- setSnapBackValue:(float)val;
- takeSnapBackValueFrom:sender;

// Dash interval (set in degrees or points depending on displayMode)
- (float)dashInterval;
- setDashInterval:(float)val;
- takeDashIntervalFrom:sender;

// Showing the main dash
- (BOOL)showMainDash;
- setShowMainDash:(BOOL)flag;
- takeShowMainDashFromSwitch:sender;

// Color of the ThumbWheel
- (NXColor)color;
- setColor:(NXColor)color;
- takeColorFrom:sender;

// tag and disabled
- takeTagFrom:sender;
- takeDisabledFromSwitch:sender;

- (const char *)getInspectorClassName;

@end

