
#import "WWMovieVolumeView.h"

@implementation WWMovieVolumeView

- initFrame:(const NXRect *)r 
{
   [super initFrame:r];
   frameStride = 1;
   offsetInX = 1;
   offsetInY = 1;

   return self;
}

- awake
{
   [super awake];
   frameStride = 1;
   offsetInX = 1;
   offsetInY = 1;

   return self;
}

- setAnimDir:(const char *)filename
{
  NXSize   imageSize;
  NXImage  *newImage;
  NXPoint  p = {0.0, 0.0};
  int      i;
  NXRect   theRect;


  if (![super setAnimDir:filename])
  {  return nil;
  }

  // okay, we've successfully loaded some movie in
  // we now want to generate a background image which
  // corresponds to a stride through the volme of 
  // the movie.

  // start off with the width and height of the movieImage
  [movieImage getSize:&imageSize];
  theRect.origin.x = 0.0;
  theRect.origin.y = 0.0;
  theRect.size.width = imageSize.width;
  theRect.size.height = imageSize.height;

  imageSize.width = imageSize.width + (offsetInX * (frameCount / frameStride));
  imageSize.height = imageSize.height + (offsetInY * (frameCount / frameStride));

  newImage = [[NXImage alloc] init];
  [newImage setSize:&imageSize];
  
  // so we want to first clear out the image, and then start
  // compositing the movie in, from back to front, offsetting the frame
  // rectangle each time
  if (![newImage lockFocus])
  {  NXLogError("unable to lockFocus on new image for WWMovieVolumeView; couldn't make a new volume image.");
     return nil;
  }

  PScompositerect(0, 0, imageSize.width, imageSize.height, NX_CLEAR);

  // for now, we'll only allow from top right down to bottom left as the volume direction
  p.x = offsetInX * (frameCount / frameStride);
  p.y = offsetInY * (frameCount / frameStride);
  for (i = (frameCount-1); i >= 0; i -= frameStride)
  {  [[imageList objectAt:i] composite:NX_SOVER toPoint:&p]; 
     p.x -= offsetInX;
     p.y -= offsetInY;
  }
  [newImage unlockFocus];
  
  // the image has to be beneath or it will just look stupid...
  [self setImageUnder:YES];
  [self setImage:newImage];

  return self;
}


@end
