
#import "WWSlider.h"
#import "WWSliderCell.h"

#import "avoidStupidWarnings.h"

@implementation WWSlider

static id myStoredCellClass;


+ initialize 
{ 
  [WWSlider setVersion:2]; 

  if (self == [WWSlider class])
  {  myStoredCellClass = [WWSliderCell class];
  }

  return self; 
}
//
+ setCellClass:classID
{
  myStoredCellClass = classID;
  return self;
}
////////////////////////////////////////////////////////
//
- initFrame:(const NXRect *)frameRect
{
  id  oldCell;
  const char  *dragTypes[] = { NXColorPboardType, NULL};


  [super initFrame:frameRect];
  [self registerForDraggedTypes:dragTypes count:1];
  oldCell = [self setCell:[[myStoredCellClass alloc] init]];
  [oldCell free];

  return self;
}


- awake
{
  const char  *dragTypes[] = { NXColorPboardType, NULL};


  [super awake];
  [self registerForDraggedTypes:dragTypes count:1];
  return self;
}

- free
{
   const char *fileType[] = { "" };
   Pasteboard *pboard = [Pasteboard newName:NXDragPboard];

   // why doesn't this work?
   [pboard declareTypes:fileType num:0 owner:nil];
   [self unregisterDraggedTypes];

   return [super free];
}

//
- awakeFromNib
{
  [minText setFloatValue:[self minValue]];
  [maxText setFloatValue:[self maxValue]];
  [valText setFloatValue:[self floatValue]];
  return self;
}
//
- setMinValue:(double)aDouble
{
   [minText setFloatValue:aDouble];
   return [super setMinValue:aDouble];
}
//
- setMaxValue:(double)aDouble
{
   [maxText setFloatValue:aDouble];
   return [super setMaxValue:aDouble];
}

// need this because there is no equivalent to awakeFromNib inside of IB...
static BOOL oTheHorrorOfItAll = YES;

- sendAction:(SEL)theAction to:theTarget
{
  if (oTheHorrorOfItAll)
  {  oTheHorrorOfItAll = NO;
     [minText setFloatValue:[self minValue]];
     [maxText setFloatValue:[self maxValue]];
  }
  [valText setFloatValue:[self floatValue]];
  if ([cell respondsTo:@selector(updateInterp)])
  {  [cell updateInterp];
  }

  return [super sendAction:theAction to:theTarget];
}

- setMinText:sender {  minText = sender; return self;}
- setMaxText:sender {  maxText = sender; return self;}
- setValText:sender {  valText = sender; return self;}

- setStringValue:(const char *)str
{
  char  *theStr;

  [super setStringValue:str];
  theStr = (char *)[self stringValue];
  [minText setStringValue:theStr];
  [maxText setStringValue:theStr];
  [valText setStringValue:theStr];
  return self;
}

- setIntValue:(int)val
{
  int   theVal;


  [super setIntValue:val];
  theVal = [self intValue];
  [minText setIntValue:theVal];
  [maxText setIntValue:theVal];
  [valText setIntValue:theVal];
  return self;
}

- setFloatValue:(float)val
{
  float  theVal;


  [super setFloatValue:val];
  theVal = [self floatValue];
  [minText setFloatValue:theVal];
  [maxText setFloatValue:theVal];
  [valText setFloatValue:theVal];
  return self;
}

// need to be able to see the cell's tclVar...
- setTclVar:(const char *)str { return [[self cell] setTclVar:str]; }
- (const char *)tclVar { return [[self cell] tclVar]; }

// need to be able to see the cell's controlString...
- setControlString:(const char *)str { return [[self cell] setControlString:str]; }
- (const char *)controlString { return [[self cell] controlString]; }

// need to be able to see the cell's tclExpression...
- setTclExpression:(const char *)str { return [[self cell] setTclExpression:str]; }
- (const char *)tclExpression { return [[self cell] tclExpression]; }

- setInterp:newInterp {  return [[self cell] setInterp:newInterp]; }
- evaluateSelf { return [[self cell] evaluateSelf]; }


// drag and drop color stuff...
- (NXDragOperation)draggingEntered:sender  {  return NX_DragOperationGeneric; }
- (NXDragOperation)draggingUpdated:sender  {  return NX_DragOperationGeneric; }
- draggingExited:sender { return self;  }
- (BOOL)prepareForDragOperation:sender {  return YES; }

- (BOOL)performDragOperation:sender
{
   Pasteboard *pboard = [Pasteboard newName:NXDragPboard];
   NXColor  aColor;

   aColor = NXReadColorFromPasteboard(pboard);
   [cell setColor:aColor];
   [self display];

   return YES;
}

- (NXColor)color { return [cell color];  }
- takeColor:sender {  [cell setColor:[sender color]]; return self; }
- setColor:(NXColor)aColor {  [cell setColor:aColor]; return self; }

// IB stuff
- (const char *)getInspectorClassName 
{  
   NXEvent  *event = [NXApp currentEvent];

   if (event->flags & NX_ALTERNATEMASK)
   {  return [super getInspectorClassName];
   }
   
   return "WWSliderIBInspector"; 
}

- write:(NXTypedStream *)stream 
{
   [super write:stream];
   return self;
}
//
- read:(NXTypedStream *)stream 
{
   int version;

   [super read:stream];

   version = NXTypedStreamClassVersion(stream, "WWSlider");
   if (version == 1)
   {  
   }
   return self;
}

@end
