/* Copyright 1993 Michael B. Johnson
 * Permission to use, copy, modify, and distribute this
 * software and its documentation for any non-commercial 
 * purpose and without fee is hereby granted, provided that the 
 * above copyright notice appears in all copies.  Michael B. Johnson
 * makes no representations about the suitability of this
 * software for any purpose.  It is provided "as is" without
 * express or implied warranty.
 *
 * Permission to use, copy, modify or distribute this software
 * and its documentation for any commercial purpose must be
 * confirmed in writing with Michael B. Johnson.  He can be
 * contacted at:
 *              20 Ames St. E15-023G    
 *              Cambridge, MA 02141
 *              (617) 547 0563
 *              
 */


#import "WWTCLInterp.h"

#import "WWTCLVarTrace.h"
#import "WWTCLClosedCmd.h"

// for isTestingInterface
#import <apps/InterfaceBuilder.h>

// for example control string stuff...
#import "WWSliderCell.h"
// for example expression stuff...
#import "WWButtonCell.h"


// for string stuff (duh!)
#import "WWSimpleString.h"

#import <ctype.h>
#import <mach-o/rld.h>

@implementation WWTCLInterp

+ initialize { [WWTCLInterp setVersion:4]; return self; }

int
cmd_loadControlPanel(WWTCLInterp *me, Tcl_Interp *interp, int argc, char **argv)
{
  int  i = 1;

  while (i < argc)
  {  if (![me loadControlPanel:argv[i++] modally:NO]) {  sprintf(interp->result, "0"); return TCL_OK; }
  }
  sprintf(interp->result, "1");
  return TCL_OK;
}
//
int
cmd_loadControlPanelModallyAndClose(WWTCLInterp *me, Tcl_Interp *interp, int argc, char **argv)
{
  int  i = 1;


  while (i < argc)
  {  if (![me loadControlPanel:argv[i++] modally:YES]) {  sprintf(interp->result, "0"); return TCL_OK; }
  }
  sprintf(interp->result, "1");
  return TCL_OK;
}

static char errBuf[1024]; 

int
cmd_deminiaturize(WWTCLInterp *me, Tcl_Interp *interp, int argc, char **argv)
{
  char  *my_args = "controlPanelTitle";
  id    controlPanel;


  if (argc != 2)
  {  sprintf(errBuf, "USAGE: %s %s", argv[0], my_args);
     Tcl_AppendResult(interp, errBuf, (char *)NULL);
     return TCL_ERROR;
  }

  controlPanel = [me findControlPanelTitled:argv[1]];
  if (controlPanel)
  {  [controlPanel deminiaturize:me];
  }
  else
  {  sprintf(errBuf, "unknown control panel named %s", argv[1]);
     Tcl_AppendResult(interp, errBuf, (char *)NULL);
     return TCL_ERROR;
  }

  return TCL_OK;
}

int
cmd_faxPSCode(WWTCLInterp *me, Tcl_Interp *interp, int argc, char **argv)
{
  char  *my_args = "controlPanelTitle";
  id    controlPanel;


  if (argc != 2)
  {  sprintf(errBuf, "USAGE: %s %s", argv[0], my_args);
     Tcl_AppendResult(interp, errBuf, (char *)NULL);
     return TCL_ERROR;
  }

  controlPanel = [me findControlPanelTitled:argv[1]];
  if (controlPanel)
  {  [controlPanel faxPSCode:me];
  }
  else
  {  sprintf(errBuf, "unknown control panel named %s", argv[1]);
     Tcl_AppendResult(interp, errBuf, (char *)NULL);
     return TCL_ERROR;
  }

  return TCL_OK;
}

int
cmd_makeKeyAndOrderFront(WWTCLInterp *me, Tcl_Interp *interp, int argc, char **argv)
{
  char  *my_args = "controlPanelTitle";
  id    controlPanel;


  if (argc != 2)
  {  sprintf(errBuf, "USAGE: %s %s", argv[0], my_args);
     Tcl_AppendResult(interp, errBuf, (char *)NULL);
     return TCL_ERROR;
  }

  controlPanel = [me findControlPanelTitled:argv[1]];
  if (controlPanel)
  {  [controlPanel makeKeyAndOrderFront:me];
  }
  else
  {  sprintf(errBuf, "unknown control panel named %s", argv[1]);
     Tcl_AppendResult(interp, errBuf, (char *)NULL);
     return TCL_ERROR;
  }

  return TCL_OK;
}

int
cmd_miniaturize(WWTCLInterp *me, Tcl_Interp *interp, int argc, char **argv)
{
  char  *my_args = "controlPanelTitle";
  id    controlPanel;


  if (argc != 2)
  {  sprintf(errBuf, "USAGE: %s %s", argv[0], my_args);
     Tcl_AppendResult(interp, errBuf, (char *)NULL);
     return TCL_ERROR;
  }

  controlPanel = [me findControlPanelTitled:argv[1]];
  if (controlPanel)
  {  [controlPanel miniaturize:me];
  }
  else
  {  sprintf(errBuf, "unknown control panel named %s", argv[1]);
     Tcl_AppendResult(interp, errBuf, (char *)NULL);
     return TCL_ERROR;
  }

  return TCL_OK;
}

int
cmd_orderBack(WWTCLInterp *me, Tcl_Interp *interp, int argc, char **argv)
{
  char  *my_args = "controlPanelTitle";
  id    controlPanel;


  if (argc != 2)
  {  sprintf(errBuf, "USAGE: %s %s", argv[0], my_args);
     Tcl_AppendResult(interp, errBuf, (char *)NULL);
     return TCL_ERROR;
  }

  controlPanel = [me findControlPanelTitled:argv[1]];
  if (controlPanel)
  {  [controlPanel orderBack:me];
  }
  else
  {  sprintf(errBuf, "unknown control panel named %s", argv[1]);
     Tcl_AppendResult(interp, errBuf, (char *)NULL);
     return TCL_ERROR;
  }

  return TCL_OK;
}

int
cmd_orderFront(WWTCLInterp *me, Tcl_Interp *interp, int argc, char **argv)
{
  char  *my_args = "controlPanelTitle";
  id    controlPanel;


  if (argc != 2)
  {  sprintf(errBuf, "USAGE: %s %s", argv[0], my_args);
     Tcl_AppendResult(interp, errBuf, (char *)NULL);
     return TCL_ERROR;
  }

  controlPanel = [me findControlPanelTitled:argv[1]];
  if (controlPanel)
  {  [controlPanel orderFront:me];
  }
  else
  {  sprintf(errBuf, "unknown control panel named %s", argv[1]);
     Tcl_AppendResult(interp, errBuf, (char *)NULL);
     return TCL_ERROR;
  }

  return TCL_OK;
}

int
cmd_orderOut(WWTCLInterp *me, Tcl_Interp *interp, int argc, char **argv)
{
  char  *my_args = "controlPanelTitle";
  id    controlPanel;


  if (argc != 2)
  {  sprintf(errBuf, "USAGE: %s %s", argv[0], my_args);
     Tcl_AppendResult(interp, errBuf, (char *)NULL);
     return TCL_ERROR;
  }

  controlPanel = [me findControlPanelTitled:argv[1]];
  if (controlPanel)
  {  [controlPanel orderOut:me];
  }
  else
  {  sprintf(errBuf, "unknown control panel named %s", argv[1]);
     Tcl_AppendResult(interp, errBuf, (char *)NULL);
     return TCL_ERROR;
  }

  return TCL_OK;
}

int
cmd_performClose(WWTCLInterp *me, Tcl_Interp *interp, int argc, char **argv)
{
  char  *my_args = "controlPanelTitle";
  id    controlPanel;


  if (argc != 2)
  {  sprintf(errBuf, "USAGE: %s %s", argv[0], my_args);
     Tcl_AppendResult(interp, errBuf, (char *)NULL);
     return TCL_ERROR;
  }

  controlPanel = [me findControlPanelTitled:argv[1]];
  if (controlPanel)
  {  [controlPanel performClose:me];
  }
  else
  {  sprintf(errBuf, "unknown control panel named %s", argv[1]);
     Tcl_AppendResult(interp, errBuf, (char *)NULL);
     return TCL_ERROR;
  }

  return TCL_OK;
}

int
cmd_performMiniaturize(WWTCLInterp *me, Tcl_Interp *interp, int argc, char **argv)
{
  char  *my_args = "controlPanelTitle";
  id    controlPanel;


  if (argc != 2)
  {  sprintf(errBuf, "USAGE: %s %s", argv[0], my_args);
     Tcl_AppendResult(interp, errBuf, (char *)NULL);
     return TCL_ERROR;
  }

  controlPanel = [me findControlPanelTitled:argv[1]];
  if (controlPanel)
  {  [controlPanel performMiniaturize:me];
  }
  else
  {  sprintf(errBuf, "unknown control panel named %s", argv[1]);
     Tcl_AppendResult(interp, errBuf, (char *)NULL);
     return TCL_ERROR;
  }

  return TCL_OK;
}

int
cmd_printPSCode(WWTCLInterp *me, Tcl_Interp *interp, int argc, char **argv)
{
  char  *my_args = "controlPanelTitle";
  id    controlPanel;


  if (argc != 2)
  {  sprintf(errBuf, "USAGE: %s %s", argv[0], my_args);
     Tcl_AppendResult(interp, errBuf, (char *)NULL);
     return TCL_ERROR;
  }

  controlPanel = [me findControlPanelTitled:argv[1]];
  if (controlPanel)
  {  [controlPanel printPSCode:me];
  }
  else
  {  sprintf(errBuf, "unknown control panel named %s", argv[1]);
     Tcl_AppendResult(interp, errBuf, (char *)NULL);
     return TCL_ERROR;
  }

  return TCL_OK;
}

int
cmd_smartFaxPSCode(WWTCLInterp *me, Tcl_Interp *interp, int argc, char **argv)
{
  char  *my_args = "controlPanelTitle";
  id    controlPanel;


  if (argc != 2)
  {  sprintf(errBuf, "USAGE: %s %s", argv[0], my_args);
     Tcl_AppendResult(interp, errBuf, (char *)NULL);
     return TCL_ERROR;
  }

  controlPanel = [me findControlPanelTitled:argv[1]];
  if (controlPanel)
  {  [controlPanel smartFaxPSCode:me];
  }
  else
  {  sprintf(errBuf, "unknown control panel named %s", argv[1]);
     Tcl_AppendResult(interp, errBuf, (char *)NULL);
     return TCL_ERROR;
  }

  return TCL_OK;
}

int
cmd_smartPrintPSCode(WWTCLInterp *me, Tcl_Interp *interp, int argc, char **argv)
{
  char  *my_args = "controlPanelTitle";
  id    controlPanel;


  if (argc != 2)
  {  sprintf(errBuf, "USAGE: %s %s", argv[0], my_args);
     Tcl_AppendResult(interp, errBuf, (char *)NULL);
     return TCL_ERROR;
  }

  controlPanel = [me findControlPanelTitled:argv[1]];
  if (controlPanel)
  {  [controlPanel smartPrintPSCode:me];
  }
  else
  {  sprintf(errBuf, "unknown control panel named %s", argv[1]);
     Tcl_AppendResult(interp, errBuf, (char *)NULL);
     return TCL_ERROR;
  }

  return TCL_OK;
}

// IB responds to these two methods, and we need to use them...
// a more heinous hacker than I figured this out...
- activeDir { return nil; }
- getCString:(char *)str maxLength:(int)pathLen { return nil; }

- init
{
  id  activeDir;


  [super init];

  interp = Tcl_CreateInterp();
  [self addCommand:"loadControlPanel" :(Tcl_CmdProc *)cmd_loadControlPanel :self];
  [self addCommand:"loadControlPanelModallyAndClose" :(Tcl_CmdProc *)cmd_loadControlPanelModallyAndClose :self];
  [self addCommand:"deminiaturize" :(Tcl_CmdProc *)cmd_deminiaturize :self];
  [self addCommand:"faxPSCode" :(Tcl_CmdProc *)cmd_faxPSCode :self];
  [self addCommand:"makeKeyAndOrderFront" :(Tcl_CmdProc *)cmd_makeKeyAndOrderFront :self];
  [self addCommand:"miniaturize" :(Tcl_CmdProc *)cmd_miniaturize :self];
  [self addCommand:"orderBack" :(Tcl_CmdProc *)cmd_orderBack :self];
  [self addCommand:"orderFront" :(Tcl_CmdProc *)cmd_orderFront :self];
  [self addCommand:"orderOut" :(Tcl_CmdProc *)cmd_orderOut :self];
  [self addCommand:"performClose" :(Tcl_CmdProc *)cmd_performClose :self];
  [self addCommand:"performMiniaturize" :(Tcl_CmdProc *)cmd_performMiniaturize :self];
  [self addCommand:"printPSCode" :(Tcl_CmdProc *)cmd_printPSCode :self];
  [self addCommand:"smartFaxPSCode" :(Tcl_CmdProc *)cmd_smartFaxPSCode :self];
  [self addCommand:"smartPrintPSCode" :(Tcl_CmdProc *)cmd_smartPrintPSCode :self];

  Tcl_InitHashTable(&uiHashTableByWindow, TCL_ONE_WORD_KEYS);

  controlPanelList = [[List alloc] init];
  controlPanel = nil;
  cmdBufSize = 8192;
  cmdBuf = (char *)NXZoneCalloc([self zone], cmdBufSize, sizeof(char));
  checkCmdBufSize = NO;

  pathInIB = (char *)NXZoneCalloc([self zone], (MAXPATHLEN+1), sizeof(char));
  // WAVE: should really load in a different IB inspector if the version of IB I'm in doesn't respond to the following...
  if ([NXApp respondsTo:@selector(activeDir)] && [NXApp respondsTo:@selector(getCString:maxLength:)]) // need to be better about this...
  {  //NXLogError("trying to ask IB where we are...");
     if ((activeDir = [NXApp activeDir]))
     {  [activeDir getCString:pathInIB maxLength:MAXPATHLEN];
        //NXLogError("IB says we're in: <%s>\n", pathInIB);
     }
     else
     {  NXLogError("Rats! IB won't tell us where we are.  You'll have to fill in pathInIB yourself...\n");
     }
  }

  nibNameArgc = 0;
  nibNameArgv = (char **)NXZoneMalloc([self zone], 256 * sizeof(char *));

  tmpStringc = 32; // some reasonable little number...
  tmpStringIndex = 0;
  tmpStringv = (char **)NXZoneMalloc([self zone], ((tmpStringc * sizeof(char *))));

  awakeInIB = NO;
  dejaVuInIB = NO;
  showErrors = YES;
  
  delegate = nil;
  errorWindow = nil;

  traceList = [[List alloc] init];
  manipulatorList = [[List alloc] init];

  return self;
}

//
- awake
{
  int  i, ret;
  id  activeDir;


  [super awake];

  interp = Tcl_CreateInterp();
  [self addCommand:"loadControlPanel" :(Tcl_CmdProc *)cmd_loadControlPanel :self];
  [self addCommand:"loadControlPanelModallyAndClose" :(Tcl_CmdProc *)cmd_loadControlPanelModallyAndClose :self];
  [self addCommand:"deminiaturize" :(Tcl_CmdProc *)cmd_deminiaturize :self];
  [self addCommand:"faxPSCode" :(Tcl_CmdProc *)cmd_faxPSCode :self];
  [self addCommand:"makeKeyAndOrderFront" :(Tcl_CmdProc *)cmd_makeKeyAndOrderFront :self];
  [self addCommand:"miniaturize" :(Tcl_CmdProc *)cmd_miniaturize :self];
  [self addCommand:"orderBack" :(Tcl_CmdProc *)cmd_orderBack :self];
  [self addCommand:"orderFront" :(Tcl_CmdProc *)cmd_orderFront :self];
  [self addCommand:"orderOut" :(Tcl_CmdProc *)cmd_orderOut :self];
  [self addCommand:"performClose" :(Tcl_CmdProc *)cmd_performClose :self];
  [self addCommand:"performMiniaturize" :(Tcl_CmdProc *)cmd_performMiniaturize :self];
  [self addCommand:"printPSCode" :(Tcl_CmdProc *)cmd_printPSCode :self];
  [self addCommand:"smartFaxPSCode" :(Tcl_CmdProc *)cmd_smartFaxPSCode :self];
  [self addCommand:"smartPrintPSCode" :(Tcl_CmdProc *)cmd_smartPrintPSCode :self];

  Tcl_InitHashTable(&uiHashTableByWindow, TCL_ONE_WORD_KEYS);

  controlPanelList = [[List alloc] init];
  controlPanel = nil;
  cmdBufSize = 8192;
  cmdBuf = (char *)NXZoneCalloc([self zone], cmdBufSize, sizeof(char));
  checkCmdBufSize = NO;

  tmpStringc = 32; // some reasonable little number...
  tmpStringIndex = 0;
  tmpStringv = (char **)NXZoneMalloc([self zone], ((tmpStringc * sizeof(char *))));

  showErrors = YES;
  errorWindow = nil;

  traceList = [[List alloc] init];
  manipulatorList = [[List alloc] init];

  // need to reset my archived variable values and proc definitions
  for (i = 0; i < setc; i++)
  {  //NXLogError("parsing <%s>\n", setv[i]);
     ret = Tcl_GlobalEval(interp, setv[i]); 
     if (ret != TCL_OK)
     {  NXLogError("warning: error parsing <%s>\n", setv[i]);
     }
     Tcl_ResetResult(interp); 
  }

  // need to reset my archived variable values and proc definitions
  for (i = 0; i < procc; i++)
  {  //NXLogError("parsing <%s>\n", procv[i]);
     ret = Tcl_GlobalEval(interp, procv[i]); 
     if (ret != TCL_OK)
     {  NXLogError("warning: error parsing <%s>\n", procv[i]);
     }
     Tcl_ResetResult(interp); 
  }

  pathInIB = (char *)NXZoneCalloc([self zone], (MAXPATHLEN+1), sizeof(char));
  if ([NXApp respondsTo:@selector(activeDir)])  // we're in IB
  {  //NXLogError("trying to ask IB where we are...");
     if ((activeDir = [NXApp activeDir]))
     {  [activeDir getCString:pathInIB maxLength:MAXPATHLEN];
        //NXLogError("IB says we're in: <%s>\n", pathInIB);
     }
     else
     {  //NXLogError("Rats! It won't tell us.\n");
     }
  }

  // if we're running in IB, we only want to load our nib files
  // when we're in test interface mode.
  if ([NXApp respondsTo:@selector(isTestingInterface)])
  {  return self;
  }

  [self loadNibFiles];

  return self;
}

- loadNibFiles
{
  char  errBuf[1024], path[MAXPATHLEN + 1];
  id    bundle;
  int   i;


  // note that the first time awake is called in IB (as you drag the 
  // object from the palette), nibNameArgc will be 0, so we won't have 
  // any problems. As you inspect the object IB and add nib filenames, 
  // and then go into test interface mode, this will actually do something...
  // also note that the only way to get this to work in IB is to use my "pathInIB"
  // hack, which basically let's this object find the absolute pathname for the 
  // nib files.  Since there doesn't seem to be any nice "bundle" way to find
  // this info out otherwise...  Anybody have any bright ideas?
  if ([NXApp respondsTo:@selector(isTestingInterface)])
  {  for (i = 0; i < nibNameArgc; i++)
     {  if (!pathInIB)
        {  sprintf(errBuf, "No path (in Interface Builder) given for %s.\n", nibNameArgv[i]);
           NXRunAlertPanel("Yo!", errBuf, NULL, NULL, NULL);
        }
        sprintf(path, "%s/%s.nib", pathInIB, nibNameArgv[i]);
        if ([self loadControlPanel:path modally:NO])
        {  
        }
        else
        {  sprintf(errBuf, "problem loading %s (in Interface Builder).\n", path);
           NXRunAlertPanel("Yo!", errBuf, NULL, NULL, NULL);
        }
     }
  }
  else
  {  bundle = [NXBundle bundleForClass: [self class]];
     if (!bundle) 
     {  sprintf(errBuf, "No bundle for Class %s - unable to unarchive its nib files\n", [self name]);
        NXRunAlertPanel("Yo!", errBuf, NULL, NULL, NULL);
     }
     for (i = 0; i < nibNameArgc; i++)
     {  if (![bundle getPath:path forResource:nibNameArgv[i] ofType:"nib"])
        {  NXLogError("No path for %s.\n", nibNameArgv[i]);
           return nil;
        }
        if ([self loadControlPanel:path modally:NO])
        {  
        }
        else
        {  sprintf(errBuf, "problem loading %s.\n", path);
           NXRunAlertPanel("Yo!", errBuf, NULL, NULL, NULL);
        }
     }
  }

  return self;
}

- nibInstantiate
{
  if (dejaVuInIB)
  {  return self;
  }
  dejaVuInIB = YES;
  if ([NXApp respondsTo:@selector(isTestingInterface)])
  {  if ([NXApp isTestingInterface])
     {  [self loadNibFiles];
     }
  }
  return self;
}


- free
{
  int i, howMany = [controlPanelList count];
  id  panel;


  if (errorWindow)
  {  [self closeErrorWindow:nil];
  }
  //NXLogError("there are %d panels to close...\n", howMany);
  for (i = howMany - 1; i >= 0; i--)
  {  NX_DURING
       panel = [controlPanelList objectAt:i];
       //NXLogError("%d: sending performClose: to %s titled <%s>\n", i, [panel name], [panel title]);
       [panel performClose:self];
     NX_HANDLER
  	NXLogError("in freeing WWTCLInterp, exception [%d] raised trying to performClose on %s <%s>.\n", 
                   NXLocalHandler.code, [[controlPanelList objectAt:i] name], [[controlPanelList objectAt:i] title]);
     NX_ENDHANDLER
  }
  [controlPanelList free];
  NXZoneFree([self zone], cmdBuf);

  for (i = 0; i < nibNameArgc; i++)
  {  if (nibNameArgv[i]) {  free(nibNameArgv[i]); }
  }

  if (pathInIB) {  free(pathInIB); }

  // if wave wasn't lame, he'd figure out why the next line is wrong right now...
  //[[traceList freeObjects] free];
  [[manipulatorList freeObjects] free];

  // do this last, so traces can be removed in the perform close part...
  Tcl_DeleteInterp(interp);

  // delete any tmp strings we had to malloc up in write:
  //  for (i = 0; i < tmpStringIndex; i++)
  //  {  if (tmpStringv[i])  {  NXZoneFree([self zone], tmpStringv[i]); }
  //  }    
  //  if (tmpStringv) {  NXZoneFree([self zone], tmpStringv); }

  return [super free];
}

- addToManipulatorList:newManipulator  {   [manipulatorList addObject:newManipulator];  return self; }


- loadErrorWindow
{
  id    bundle = [NXBundle bundleForClass: [self class]];
  char  path[MAXPATHLEN + 1];


  if (!bundle) 
  {  NXLogError("No bundle for Class %s - unable to showErrorWindow\n", [self name]);
     return nil;
  }
  if (![bundle getPath:path forResource:"WWTCLInterpError" ofType:"nib"])
  {  NXLogError("couldn't find WWTCLInterpError.nib. Did you subclass WWTCLInterp and not include the nib in the palette?\n");
     return nil;
  }
  [NXApp loadNibFile:path owner:self  withNames:NO fromZone:[self zone]];
  //NXLogError ("Loaded %s.\n", path);

  return self;
}

- closeErrorWindow:sender
{
  [NXApp stopModal];
  [errorWindow close];
  errorWindow = nil;

  return self;
}


- setShowErrors:(BOOL)flag {  showErrors = flag; return self; }
- (BOOL)showErrors { return showErrors; }

- showError:(char *)error
{
  char *errorInfo;


  if (!errorWindow)
  {  [self loadErrorWindow];
  }
  if (!errorWindow)
  {  NXLogError("unable to load error window for error <%s>", error);
     return nil;
  }
  [errorInfoText setSel:0 :[errorInfoText textLength]];
  errorInfo = Tcl_GetVar(interp, "errorInfo", TCL_GLOBAL_ONLY);
  [errorInfoText replaceSel:errorInfo];
  [errorWindow center];
  //[errorWindow makeKeyAndOrderFront:self];
  [NXApp runModalFor:errorWindow];
  return self;
}


- findControlPanelTitled:(const char *)title
{
  BOOL    notFound = YES;
  int     i = 0, 
          howMany = [controlPanelList count];
  const 
    char  *aTitle;


  if (!title)
  {  return nil;
  }

  while ((notFound) && (i < howMany))
  {  aTitle = [[controlPanelList objectAt:i] title];
     if (aTitle)
     {  if (!strcmp(title, aTitle))
        {  notFound = NO;
        }
        else
        {  i++;
        }
     }
     else
     {  i++;
     }
  }

  if (notFound)
  {  return nil;
  }

  return [controlPanelList objectAt:i];
}

- (int)eval:(const char *)cmd  
{  
   int  ret;


   ret = Tcl_Eval(interp, (char *)cmd); 
   if (showErrors)
   {  if (ret == TCL_ERROR)
      {  [self showError:interp->result];
      }
   }
   Tcl_ResetResult(interp); 
   return ret;
}

// if return value != NULL, it should be free()ed by caller...
//
- (char *)eval:(const char *)cmd :(int *)ret
{
  char *result = NULL;
  int tcl_eval_ret;


  Tcl_ResetResult(interp);
  if ((tcl_eval_ret = Tcl_Eval(interp, (char *)cmd)) == TCL_OK)
  {  if (interp->result)
     {  result = NXCopyStringBuffer(interp->result);
     }
     else
     {  result = NULL;
     }
     *ret = TCL_OK;
     return result;
  }
  else
  {  if (tcl_eval_ret == TCL_ERROR)
     {  if (interp->result)
	{  result = NXCopyStringBuffer(interp->result);
        }
        else
        {  result = NULL;
        }
        if (showErrors)  // only do this for a *real* error
        { [self showError:interp->result]; 
        }
     }
     {  if (interp->result)
        {  result = NXCopyStringBuffer(interp->result);
        }
        else
        {  result = NULL;
        }
     }
     if (tcl_eval_ret == TCL_BREAK)
     {  result = NXCopyStringBuffer("The command requests that the innermost loop be exited");
     }
     if (tcl_eval_ret == TCL_CONTINUE)
     {  result = NXCopyStringBuffer("Go on to the next iteration of the current loop");
     }
  }
  Tcl_ResetResult(interp);
  *ret = tcl_eval_ret;
  return result;
}


- (int)globalEval:(const char *)cmd  
{  
   int  ret;


   ret = Tcl_GlobalEval(interp, (char *)cmd); 
   if (showErrors)
   {  if (ret == TCL_ERROR)
      {  [self showError:interp->result];
      }
   }
   Tcl_ResetResult(interp); 
   return ret;
}

// if return value != NULL, it should be free()ed by caller...
//
- (char *)globalEval:(const char *)cmd :(int *)ret
{
  char *result = NULL;
  int tcl_eval_ret;


  Tcl_ResetResult(interp);
  if ((tcl_eval_ret = Tcl_GlobalEval(interp, (char *)cmd)) == TCL_OK)
  {  if (interp->result)
     {  result = NXCopyStringBuffer(interp->result);
     }
     else
     {  result = NULL;
     }
     *ret = TCL_OK;
     return result;
  }
  else
  {  if (tcl_eval_ret == TCL_ERROR)
     {  if (interp->result)
	{  result = NXCopyStringBuffer(interp->result);
        }
        else
        {  result = NULL;
        }
        if (showErrors)  // only do this for a *real* error
        { [self showError:interp->result]; 
        }
     }
     if (tcl_eval_ret == TCL_RETURN)
     {  if (interp->result)
        {  result = NXCopyStringBuffer(interp->result);
        }
        else
        {  result = NULL;
        }
     }
     if (tcl_eval_ret == TCL_BREAK)
     {  result = NXCopyStringBuffer("The command requests that the innermost loop be exited");
     }
     if (tcl_eval_ret == TCL_CONTINUE)
     {  result = NXCopyStringBuffer("Go on to the next iteration of the current loop");
     }
  }
  Tcl_ResetResult(interp);
  *ret = tcl_eval_ret;
  return result;
}


- (void)resetResult
{
   Tcl_ResetResult(interp);
   return ;
}


- (BOOL)exprBoolean:(const char *)cmd
{
  int  result;


  if (Tcl_ExprBoolean(interp, (char *)cmd, &result) != TCL_OK)
  {  if (showErrors)  // only do this for a *real* error
     { [self showError:interp->result]; 
     }
  }
  return result;
}

- (long)exprLong:(const char *)cmd
{
  long  result;


  if (Tcl_ExprLong(interp, (char *)cmd, &result) != TCL_OK)
  {  if (showErrors)  // only do this for a *real* error
     { [self showError:interp->result]; 
     }
  }
  return result;
}

- (double)exprDouble:(const char *)cmd
{
  double  result;


  if (Tcl_ExprDouble(interp, (char *)cmd, &result) != TCL_OK)
  {  if (showErrors)  // only do this for a *real* error
     { [self showError:interp->result]; 
     }
  }
  return result;
}

// if return value != NULL, it should be free()ed by caller...
//
- (char *)exprString:(const char *)cmd :(int *)ret
{
  char *result = NULL;
  int tcl_eval_ret;


  if ((tcl_eval_ret = Tcl_ExprString(interp, (char *)cmd)) == TCL_OK)
  {  if (interp->result)
     {  result = NXCopyStringBuffer(interp->result);
     }
     else
     {  result = NULL;
     }
     Tcl_ResetResult(interp);
     *ret = TCL_OK;
     return result;
  }
  else
  {  if (tcl_eval_ret == TCL_ERROR)
     {  if (interp->result)
	{  result = NXCopyStringBuffer(interp->result);
        }
        else
        {  result = NULL;
        }
        if (showErrors)  // only do this for a *real* error
        { [self showError:interp->result]; 
        }
     }
     if (tcl_eval_ret == TCL_RETURN)
     {  if (interp->result)
        {  result = NXCopyStringBuffer(interp->result);
        }
        else
        {  result = NULL;
        }
     }
     if (tcl_eval_ret == TCL_BREAK)
     {  result = NXCopyStringBuffer("The command requests that the innermost loop be exited");
     }
     if (tcl_eval_ret == TCL_CONTINUE)
     {  result = NXCopyStringBuffer("Go on to the next iteration of the current loop");
     }
  }
  Tcl_ResetResult(interp);
  *ret = tcl_eval_ret;
  return result;
}

- (int)unsetVar:(char *)variableName 
{
  return Tcl_UnsetVar(interp, variableName, 0);
}

- (int)unsetVar2:(char *)name1 :(char *)name2
{
  return Tcl_UnsetVar2(interp, name1, name2, 0); 
}

- (char *)setVar:(char *)variableName toValue:(char *)newValue
{
  return Tcl_SetVar(interp, variableName, newValue, TCL_GLOBAL_ONLY);
}

- (char *)setVar2:(char *)name1 :(char *)name2 toValue:(char *)newValue
{
  return Tcl_SetVar2(interp, name1, name2, newValue, TCL_GLOBAL_ONLY); 
}

- (char *)setLocalVar:(char *)variableName toValue:(char *)newValue
{
  return Tcl_SetVar(interp, variableName, newValue, 0);
}

- (char *)setLocalVar2:(char *)name1 :(char *)name2 toValue:(char *)newValue
{
  return Tcl_SetVar2(interp, name1, name2, newValue, 0); 
}

- (BOOL)globalVarExists:(const char *)varName
{
  if (Tcl_GetVar(interp, (char *)varName, TCL_GLOBAL_ONLY))
  {  return YES;
  }
  return NO;
}


- (BOOL)varExists:(const char *)varName
{
  if (Tcl_GetVar(interp, (char *)varName, 0))
  {  return YES;
  }
  return NO;
}


- (char *)getVar:(char *)variableName
{
  return Tcl_GetVar(interp, variableName, TCL_GLOBAL_ONLY);
}

- (char *)getVar2:(char *)name1 :(char *)name2
{
  return Tcl_GetVar2(interp, name1, name2, TCL_GLOBAL_ONLY);
}

- (char *)getLocalVar:(char *)variableName
{
  return Tcl_GetVar(interp, variableName, 0);
}

- (char *)getLocalVar2:(char *)name1 :(char *)name2
{
  return Tcl_GetVar2(interp, name1, name2, 0); 
}

- (void)addCommand:(const char *)name :(Tcl_CmdProc *)func :me
{
   Tcl_CreateCommand(interp, (char *)name, func, (ClientData)me, NULL);
   return ;
}

- (void)addCommand:(const char *)name :(Tcl_CmdProc *)func :me :(Tcl_CmdDeleteProc *)deleteFunc
{
   Tcl_CreateCommand(interp, (char *)name, func, (ClientData)me, deleteFunc);
   return ;
}

- (BOOL)addCommand:(const char *)name fromFile:(const char *)fileName usingFunction:(const char *)funcName AndObject:me
{
    unsigned long  func;
    char           *objectFiles[2];


    objectFiles[0] = (char *)fileName;
    objectFiles[1] = NULL;

    if (!rld_load(NULL, NULL, objectFiles, NULL))
    {  NXLogError("unable to load object file %s", fileName);
       return NO;
    }
    if (!rld_lookup(NULL, funcName, &func))
    {  NXLogError("unable to lookup symbol %s", funcName);
       return NO;
    }

    Tcl_CreateCommand(interp, (char *)name, (Tcl_CmdProc *)func,
		      (ClientData)me, NULL);
    return YES;
}

- (void)createMathFunc:(const char *)name numArgs:(int)numArgs argTypes:(Tcl_ValueType *)argTypes proc:(Tcl_MathProc *)proc clientData:(ClientData)clientData
{
  Tcl_CreateMathFunc(interp,(char *)name, numArgs, argTypes, proc, clientData);
  return ;
}

- (Tcl_Interp *)interp  {  return interp; }

- (int)source:(const char *)filename
{
  char cmd[8 + MAXPATHLEN];


  sprintf(cmd, "source %s", filename);
  return [self eval:cmd];
}

/////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////
- setDelegate:newDelegate  {  delegate = newDelegate; return self; }
- delegate  {  return delegate; }

/////////////////////////////////////////////////////////////////////

// this is around for back compatibility only...really shouldn't be used, though 
static char *WriteProcForTclVar(ClientData clientData, Tcl_Interp *interp, char *name1, char *name2, int flags)
{
  [(WWTCLVarTrace *)clientData setStringValue:(const char *)Tcl_GetVar2(interp, name1, name2, flags & TCL_GLOBAL_ONLY)]; 
  // am I leaking here?
  return NULL;
}

static char *WriteProcForTclExpression(ClientData clientData, Tcl_Interp *interp, char *name1, char *name2, int flags)
{
  [(WWTCLVarTrace *)clientData executeTrace];
  return NULL;
}


static int notJustBlanks(const char *str)
{
  const char  *ptr;


  ptr = str;
  if (ptr)
  {  while (*ptr && isspace(*ptr))
     {  ptr++;
     }
     if (*ptr) // if we're not at the end and we stopped, it's not just blanks...
     {  return 1;
     }
  }
  return 0;
}


- updateTclVariables:view :list
{
  int         count, count2, i, j, varCnt;
  const char  *aTclVar, *anExpression;
  char        *tmpValue, **varV;
  id          aCell, theCellList;
  BOOL        gotIt;


  count = [[view subviews] count];
  if (count)
  {  for (i = 0; i < count; i++)
     {  [self updateTclVariables:[[view subviews] objectAt:i] :list];
     }
  }
  else
  {  // note we're changing things here.
     // we want to see if the view responds to the selector "expression"
     // if it does, we grab it and grovel over it, finding out what variables are in it.
     // we then set up a trace for each one.
     // we then evaluate the expression and set the string value to it...
     // nope, changed it again.  Because the most common case is one where
     // they just are interested in a single tcl variable, we make that be the
     // first one checked.

     if ([view respondsTo:@selector(setInterp:)])
     {  [view setInterp:self]; // this is so we aren't bound by target/action stuff...
                               // we also need to make sure we set this guy's interp,
                               // even if it doesn't have a valid tclVar or tclExpression
                               // Think about it; what if you have a button that has
                               // a controlString (and therefore needs to be connected
                               // to it's interp)...
     }
     if ([view respondsTo:@selector(tclVar)])
     {  aTclVar = [view tclVar];
        if (notJustBlanks(aTclVar))  
        {  tmpValue = Tcl_GetVar(interp, (char *)aTclVar, TCL_GLOBAL_ONLY);
           if (tmpValue)
           {  [view setStringValue:tmpValue];
	   }
           if (list) 
           {  [self traceWritesOn:(char *)aTclVar andCall:(Tcl_VarTraceProc *)WriteProcForTclVar usingData:(ClientData)view];
              [list addObject:view];
 	   }
           return self;
        } 
        // drop through; it responds to it, but it hasn't been set.
     }

     // try for tclExpression, since it either doesn't respond to tclVar, or else it's tclVar hasn't been set to anything...
     if ([view respondsTo:@selector(setInterp:)])
     {  [view setInterp:self]; // this is so we aren't bound by target/action stuff...
                               // we also need to make sure we set this guy's interp,
                               // even if it doesn't have a valid tclVar or tclExpression
                               // Think about it; what if you have a button that has
                               // a controlString (and therefore needs to be connected
                               // to it's interp)...
     }
     if ([view respondsTo:@selector(tclExpression)])
     {  anExpression = [view tclExpression];
        if (notJustBlanks(anExpression))
        {  [view evaluateSelf];
           if (list)
           {  varV = [self argvFromCmd:anExpression];
              varCnt = 0;
              if (varV)
              {  while (varV[varCnt])
                 {  varCnt++;
                 }
              }
              for (i = 0; i < varCnt; i++)
              {  [self traceWritesOn:varV[i] andCall:(Tcl_VarTraceProc *)WriteProcForTclExpression usingData:(ClientData)view];
                 free(varV[i]);
              }
              free(varV);
              [list addObject:view];
           }
        }
        return self;
     }

     if ([view respondsTo:@selector(cellList)])  // it's a Matrix!
     {  // need to grovel over each Cell, and see if it responds to "tclVar"
	theCellList = [view cellList];
	count2 = [theCellList count];
	for (j = 0; j < count2; j++)
	{  aCell = [theCellList objectAt:j];
           gotIt = NO;
           if ([aCell respondsTo:@selector(setInterp:)])
           {  [aCell setInterp:self]; // this is so we aren't bound by target/action stuff...
                                      // we also need to make sure we set this guy's interp,
                                      // even if it doesn't have a valid tclVar or tclExpression
                                      // Think about it; what if you have a button that has
                                      // a controlString (and therefore needs to be connected
                                      // to it's interp)...
           }
           if ([aCell respondsTo:@selector(tclVar)])
           {  aTclVar = [aCell tclVar];
              if (notJustBlanks(aTclVar))
              {  tmpValue = Tcl_GetVar(interp, (char *)aTclVar, TCL_GLOBAL_ONLY);
                 if (tmpValue)
		 {  [aCell setStringValue:tmpValue];
		 }
		 if (list)
		 {  [self traceWritesOn:aTclVar andCall:(Tcl_VarTraceProc *)WriteProcForTclVar usingData:(ClientData)aCell];
		    [list addObject:aCell];
		 }
	      }
           }
           if (!gotIt && ([aCell respondsTo:@selector(tclExpression)]))
	   {  anExpression = [aCell tclExpression];
	      if (notJustBlanks(anExpression))
              {  [aCell evaluateSelf];
	   	 if (list)
		 {  varV = [self argvFromCmd:anExpression];
		    varCnt = 0;
		    if (varV)
		    {  while (varV[varCnt])
		       {  varCnt++;
		       }
		    }
		    for (i = 0; i < varCnt; i++)
		    {  [self traceWritesOn:varV[i] andCall:(Tcl_VarTraceProc *)WriteProcForTclExpression usingData:(ClientData)aCell];
		       free(varV[i]);
		    }
                    free(varV);
		    [list addObject:aCell];
		 }
	      }
	   }
        }
     }
  }

  return self;
}

- unloadControlPanels
{
  int i, howMany = [controlPanelList count];
  id  panel;


  for (i = howMany - 1; i >= 0; i--)
  {  NX_DURING
       panel = [controlPanelList objectAt:i];
       //NXLogError("%d: sending performClose: to %s titled <%s>\n", i, [panel name], [panel title]);
       [panel performClose:self];
     NX_HANDLER
  	NXLogError("in WWTCLInterp's -unloadControlPanels method, exception [%d] raised trying to performClose on %s <%s>.\n", 
                   NXLocalHandler.code, [[controlPanelList objectAt:i] name], [[controlPanelList objectAt:i] title]);
     NX_ENDHANDLER
  }
  [manipulatorList freeObjects];
  return self;
}


- loadControlPanel:(char *)nibFilename modally:(BOOL)runModallyAndClose
{
  int            new, i, j, k, cnt, ret, varCnt;
  id             aList = nil, theList = nil, retID = nil, 
                 tmpAccessoryPanelList = nil, 
                 accessoryUILists = nil, theObject = nil;
  Tcl_HashEntry  *entryPtr;
  static char    completeNibFilename[MAXPATHLEN+1];       
  char           *modelPath, **varV;
  const char     *anExpression, *aTclVar;
  BOOL           gotIt;


  // unset controlPanel so that we know if it gets set from this nib file...
  initializationText = nil;
  controlPanel = nil;
  accessoryPanel0 = nil;
  accessoryPanel1 = nil;
  accessoryPanel2 = nil;
  accessoryPanel3 = nil;
  accessoryPanel4 = nil;
  accessoryPanel5 = nil;
  accessoryPanel6 = nil;
  accessoryPanel7 = nil;
  accessoryPanel8 = nil;
  accessoryPanel9 = nil;
  tmpAccessoryPanelList = [[List alloc] init];

  // if the first character of the nibFilename is /...
  
  if (nibFilename[0] == '/')
  {NX_DURING
     retID = [NXApp loadNibFile:nibFilename owner:self  withNames:NO fromZone:[self zone]];
   NX_HANDLER
  	NXLogError("in WWTCLInterp, exception [%d] raised trying to load nib <%s>.\n", 
                   NXLocalHandler.code, nibFilename);
   NX_ENDHANDLER
     if (retID)
     {  //NXLogError("Loaded %s.\n", nibFilename);
     }
     else
     {  NXLogError("unable to load %s.\n", nibFilename);
        [tmpAccessoryPanelList free];
        return nil;
     }
  }
  else
  {  modelPath = Tcl_GetVar(interp, "modelPath", TCL_GLOBAL_ONLY);
     if (modelPath)
     {  sprintf(completeNibFilename, "%s/", modelPath);
     }
     else
     {  completeNibFilename[0] = '\0';
     }

     sprintf((completeNibFilename + strlen(completeNibFilename)), "%s", nibFilename);
   NX_DURING
     retID = [NXApp loadNibFile:completeNibFilename owner:self  withNames:NO fromZone:[self zone]];
   NX_HANDLER
  	NXLogError("in WWTCLInterp, exception [%d] raised trying to load nib <%s>.\n", 
                   NXLocalHandler.code, nibFilename);
   NX_ENDHANDLER
     if (retID)
     {  //NXLogError("Loaded %s.\n", completeNibFilename);
     }
     else
     {  NXLogError("unable to load %s.\n", completeNibFilename);
        [tmpAccessoryPanelList free];
        return nil;
     }
  }

  // this nib file has to have the "controlPanel" set...
  // we want to grovel over all the subviews of "controlPanel" to see
  // which ones are UI elements which correspond to currently set tcl
  // variables, so that we can initialize the UI elements to the current
  // value of that given tcl variable.  The UI elements that fit the
  // description are either a matrix (which would respond to a "cellList"
  // msg, or a view that responds to the "tclVar" msg.  So... 

  if (initializationText) 
  { if ([initializationText respondsTo:@selector(textLength)])
    {  cnt = [initializationText textLength];
       if (cnt)
       {  char  *initStr = (char *)malloc(cnt+1);
        
          [initializationText getSubstring:initStr start:0 length:(cnt+1)];
          // this will show any errors on screen...
          ret = [self eval:initStr];
          free(initStr);
          if (ret)
          {  NXLogError("evaluating the initializationText produced an error [%d]- aborting evaluating this nib.\n", ret);
	     if (accessoryPanel0) { [accessoryPanel0 performClose:nil]; }
	     if (accessoryPanel1) { [accessoryPanel1 performClose:nil]; }
	     if (accessoryPanel2) { [accessoryPanel2 performClose:nil]; }
	     if (accessoryPanel3) { [accessoryPanel3 performClose:nil]; }
	     if (accessoryPanel4) { [accessoryPanel4 performClose:nil]; }
	     if (accessoryPanel5) { [accessoryPanel5 performClose:nil]; }
	     if (accessoryPanel6) { [accessoryPanel6 performClose:nil]; }
	     if (accessoryPanel7) { [accessoryPanel7 performClose:nil]; }
	     if (accessoryPanel8) { [accessoryPanel8 performClose:nil]; }
	     if (accessoryPanel9) { [accessoryPanel9 performClose:nil]; }
	     if (controlPanel) { [controlPanel performClose:nil]; }
	     //NOTE: this is bogus: I should free *all* the objects in the nib.
             return nil;
 	  }
       }
    }
    else
    {  if ([initializationText respondsTo:@selector(stringValue)])
       {  if (notJustBlanks([initializationText stringValue])) 
          {  ret = [self eval:[initializationText stringValue]];
             if (ret)
             {  NXLogError("evaluating the initializationText produced an error [%d]- aborting evaluating this nib.\n", ret);
		if (accessoryPanel0) { [accessoryPanel0 performClose:nil]; }
		if (accessoryPanel1) { [accessoryPanel1 performClose:nil]; }
		if (accessoryPanel2) { [accessoryPanel2 performClose:nil]; }
		if (accessoryPanel3) { [accessoryPanel3 performClose:nil]; }
		if (accessoryPanel4) { [accessoryPanel4 performClose:nil]; }
		if (accessoryPanel5) { [accessoryPanel5 performClose:nil]; }
		if (accessoryPanel6) { [accessoryPanel6 performClose:nil]; }
		if (accessoryPanel7) { [accessoryPanel7 performClose:nil]; }
		if (accessoryPanel8) { [accessoryPanel8 performClose:nil]; }
		if (accessoryPanel9) { [accessoryPanel9 performClose:nil]; }
		if (controlPanel) { [controlPanel performClose:nil]; }
		//NOTE: this is bogus: I should free *all* the objects in the nib.
                return nil;
 	     }
	  }
       }
       else    
       {  NXLogError("the object connected to the outlet <initializationText> is of class <%s>, which doesn't respond to -textLength or stringValue", 
		     [[initializationText class] name]);
       }
    }
  }

  // this hack is for accessoryPanels, i.e. panels that have
  // (potentially) tcl UI elements, but aren't the main thang.  I'm too
  // lazy to write my own IB ConnectionInspector, so we have this hack...
  if (accessoryPanel0) { [tmpAccessoryPanelList addObject:accessoryPanel0]; }
  if (accessoryPanel1) { [tmpAccessoryPanelList addObject:accessoryPanel1]; }
  if (accessoryPanel2) { [tmpAccessoryPanelList addObject:accessoryPanel2]; }
  if (accessoryPanel3) { [tmpAccessoryPanelList addObject:accessoryPanel3]; }
  if (accessoryPanel4) { [tmpAccessoryPanelList addObject:accessoryPanel4]; }
  if (accessoryPanel5) { [tmpAccessoryPanelList addObject:accessoryPanel5]; }
  if (accessoryPanel6) { [tmpAccessoryPanelList addObject:accessoryPanel6]; }
  if (accessoryPanel7) { [tmpAccessoryPanelList addObject:accessoryPanel7]; }
  if (accessoryPanel8) { [tmpAccessoryPanelList addObject:accessoryPanel8]; }
  if (accessoryPanel9) { [tmpAccessoryPanelList addObject:accessoryPanel9]; }

  if (controlPanel)
  {  aList = nil;
     accessoryUILists = nil; 

     accessoryUILists = [[List alloc] init];
     for (i = 0; i < [tmpAccessoryPanelList count]; i++)
     {  [accessoryUILists addObject:[[List alloc] init]];
     }
     aList = [[List alloc] init];

     for (i = 0; i < [tmpAccessoryPanelList count]; i++)
     {  [self updateTclVariables:[[tmpAccessoryPanelList objectAt:i] contentView] :[accessoryUILists objectAt:i]];
     }
     [self updateTclVariables:[controlPanel contentView] :aList];

     // this sets up the variable traces (and, therefore, the
     // interdependent constraints on the values of the UI widgets)

     if (runModallyAndClose)
     {  continueControlPanel = YES;

        // need to update values first...

        [NXApp runModalFor:controlPanel];
        // need to grovel over the list and remove the variable traces...
        for (i = 0; i < [accessoryUILists count]; i++)
        {  theList = [accessoryUILists objectAt:i];
           for (j = 0; j < [theList count]; j++)
	   {  theObject = [theList objectAt:j];
              gotIt = NO;
	      if ([theObject respondsTo:@selector(tclVar)])
	      {  aTclVar = [theObject tclVar];
		 if (notJustBlanks(aTclVar))
   	         {  [self untraceWritesOn:(const char *)aTclVar calling:(Tcl_VarTraceProc *)WriteProcForTclVar usingData:(ClientData)theObject];
                    gotIt = YES;
	         }
              }
	      if (!gotIt && [theObject respondsTo:@selector(tclExpression)])
	      {  anExpression = [theObject tclExpression];
		 if (notJustBlanks(anExpression))
		 {  varV = [self argvFromCmd:anExpression];
		    varCnt = 0;
		    if (varV)
		    {  while (varV[varCnt])
		       {  varCnt++;
		       }
		    }
		    for (k = 0; k < varCnt; k++)
		    {  [self untraceWritesOn:varV[k] calling:(Tcl_VarTraceProc *)WriteProcForTclExpression usingData:(ClientData)theObject];
		       free(varV[k]);
		    }
		    free(varV);
		 }
	      }
	   }
        }
        [accessoryUILists freeObjects];
        [accessoryUILists free];
       
	for (j = 0; j < [aList count]; j++)
	{  theObject = [aList objectAt:j];
           gotIt = NO;
           if ([theObject respondsTo:@selector(tclVar)])
	   {  aTclVar = [theObject tclVar];
	      if (notJustBlanks(aTclVar))
   	      {  [self untraceWritesOn:(const char *)aTclVar calling:(Tcl_VarTraceProc *)WriteProcForTclVar usingData:(ClientData)theObject];
                 gotIt = YES;
	      }
           }
	   if (!gotIt && [theObject respondsTo:@selector(tclExpression)])
	   {  anExpression = [theObject tclExpression];
	      if (notJustBlanks(anExpression))
	      {  varV = [self argvFromCmd:anExpression];
		 varCnt = 0;
		 if (varV)
		 {  while (varV[varCnt])
		    {  varCnt++;
		    }
		 }
		 for (k = 0; k < varCnt; k++)
		 {  [self untraceWritesOn:varV[k] calling:(Tcl_VarTraceProc *)WriteProcForTclExpression usingData:(ClientData)theObject];
		    free(varV[k]);
		 }
		 free(varV);
	      }
	   }
        }
        [aList free];
        [accessoryPanel0 close];
        [accessoryPanel1 close];
        [accessoryPanel2 close];
        [accessoryPanel3 close];
        [accessoryPanel4 close];
        [accessoryPanel5 close];
        [accessoryPanel6 close];
        [accessoryPanel7 close];
        [accessoryPanel8 close];
        [accessoryPanel9 close];
        [tmpAccessoryPanelList free];
        if (!continueControlPanel)
        {  return nil;
        }
     }
     else // hang on to it
     {  [controlPanelList addObject:controlPanel];  // hold on to control panels regardless so they can be raised and lowered
        if ([aList count])  // were there any tcl UI elements?
        {  [controlPanel setDelegate:self];
           new = 0;
           entryPtr = Tcl_CreateHashEntry(&uiHashTableByWindow, (char *)controlPanel, &new);
           if (!new)
           {  NXLogError("Yikes! Unable to hash the UI elements associated with the controlPanel.\n");
              [tmpAccessoryPanelList free];
              return nil;
           }
           Tcl_SetHashValue(entryPtr, (ClientData)aList);
        }

        for (i = 0; i < [accessoryUILists count]; i++)
        {  [controlPanelList addObject:[tmpAccessoryPanelList objectAt:i]];  // hold on to them for raising and lowering...
           if ([[accessoryUILists objectAt:i] count])  // were there any tcl UI elements?
           {  [[tmpAccessoryPanelList objectAt:i] setDelegate:self];
              new = 0;
              entryPtr = Tcl_CreateHashEntry(&uiHashTableByWindow, (char *)[tmpAccessoryPanelList objectAt:i], &new);
              if (!new)
              {  NXLogError("Yikes! Unable to hash the UI elements associated with one of the accessory panels.\n");
                 [tmpAccessoryPanelList free];
                 return nil;
              }
              Tcl_SetHashValue(entryPtr, (ClientData)[accessoryUILists objectAt:i]);
           }
        }
     }
  }
  return self;
}

- cancelControlPanel:sender
{
  continueControlPanel = NO;
  [NXApp stopModal];
  [controlPanel close]; 
  return self;
}


- continueControlPanel:sender
{
  continueControlPanel = YES;
  [NXApp stopModal];
  [controlPanel close];  
  return self;
}


- (BOOL)regExpMatch:(char *)string against:(char *)regexp
{
  if (Tcl_RegExpMatch(interp, string, regexp))
  {  return YES;
  }
  return NO;
}


#define MAXARGLEN 128
- (char **)argvFromCmd:(const char *)cmd
{
  char  **argv, *ptr, tmpArg[MAXARGLEN];  // yea, this is bad, evil, and nasty...
  int   i, index, len, cnt, howMany, argvLen;   // but if you name a variable longer than that, you're an ass...
  BOOL  isArrayMember;


  ptr = (char *)cmd;
  cnt = 0; 
  howMany = strlen(cmd);
  // looking for '$', 'set '

  // scan the first time, counting how many args
  // scan the next time, counting how long each is
  // scan the final time, copying the strings in
  argvLen = 0;
  while ((ptr-cmd) < howMany)
  {
     if (*ptr == '$')
     {  argvLen++; ptr++; 
        isArrayMember = NO;
        // it might have a { right after $
        if (*ptr == '{')
	{  ptr++; while ((*ptr)  && (*ptr != '}')) { ptr++; }
	}
        else // could be in an expression, which has lots of operators that can be butted up against each other...
        {  while ((*ptr)  && !isspace(*ptr) 
                          && (*ptr != '$') 
                          && (*ptr != '+') && (*ptr != '-') && (*ptr != '*') && (*ptr != '/') && (*ptr != '%') 
                          && (*ptr != '!') && (*ptr != '~') && (*ptr != '<') && (*ptr != '>') && (*ptr != '=') 
                          && (*ptr != '&') && (*ptr != '^') && (*ptr != '|') && (*ptr != '?') && (*ptr != ':')
                          && (*ptr != '[') && (*ptr != ']') && (*ptr != '}'))
           {  if (*ptr != ')' && !isArrayMember) // shouldn't use the ')' as part of the variable name
	      {  break;
	      }
              if (*ptr != '(')
	      {  isArrayMember = YES;
 	      }
              ptr++; 
           } 
        }
     }
     else // look for set
     { ptr++;
     }
  }

  if (argvLen)
  {  if (argvLen == 1)
     {  //NXLogError("I think there is one variable in the command.\n");
     }
     else
     {  //NXLogError("I think there are %d variables in the command.\n", argvLen);
     }
  }
  else
  {  //NXLogError("I don't think there are any variables in the command.\n");
     return NULL;
  }

  argv = (char **)malloc(sizeof(char *) * (argvLen+1));
  argv[argvLen] = NULL;  // mark the end of it

  ptr = (char *)cmd;
  i = -1;
  len = 0;
  while ((ptr-cmd) < howMany)
  {  if (*ptr && (*ptr == '$'))
     {  i++; ptr++; len = 0;
        isArrayMember = NO;
        // it might have a { right after $
        if (*ptr && (*ptr == '{'))
	{  ptr++; while ((*ptr)  && (*ptr != '}')) { ptr++; len++; }
	}
        else // could be in an expression, which has lots of operators that can be butted up against each other...
        {  while ((*ptr)  && !isspace(*ptr) 
                          && (*ptr != '$') 
                          && (*ptr != '+') && (*ptr != '-') && (*ptr != '*') && (*ptr != '/') && (*ptr != '%') 
                          && (*ptr != '!') && (*ptr != '~') && (*ptr != '<') && (*ptr != '>') && (*ptr != '=') 
                          && (*ptr != '&') && (*ptr != '^') && (*ptr != '|') && (*ptr != '?') && (*ptr != ':')
                          && (*ptr != '[') && (*ptr != ']') && (*ptr != '}'))
           {  if (*ptr == ')' && !isArrayMember) // shouldn't use the ')' as part of the variable name
	      {  break;
	      }
              if (*ptr == '(')
	      {  isArrayMember = YES;
 	      }
              ptr++; len++;
           } 
        }
        len++;
        //NXLogError("arg %d has %d characters (including NULL) in it.\n", i, len);
        argv[i] = (char *)malloc(len);
     }
     else // punt set
     {  ptr++;
     }
  }

  ptr = (char *)cmd;
  i = -1;
  index = 0;
  while ((ptr-cmd) < howMany)
  {  if (*ptr && (*ptr == '$'))
     {  i++; ptr++; index = 0;
        isArrayMember = NO;
        // it might have a { right after $
        if (*ptr && (*ptr == '{'))
	{  ptr++; while ((*ptr)  && (*ptr != '}')) { tmpArg[index++] = *ptr++; }
	}
        else // could be in an expression, which has lots of operators that can be butted up against each other...
        {  while ((*ptr)  && !isspace(*ptr) 
                          && (*ptr != '$') 
                          && (*ptr != '+') && (*ptr != '-') && (*ptr != '*') && (*ptr != '/') && (*ptr != '%') 
                          && (*ptr != '!') && (*ptr != '~') && (*ptr != '<') && (*ptr != '>') && (*ptr != '=') 
                          && (*ptr != '&') && (*ptr != '^') && (*ptr != '|') && (*ptr != '?') && (*ptr != ':')
                          && (*ptr != '[') && (*ptr != ']') && (*ptr != '}'))
           {  if (*ptr == ')' && !isArrayMember) // shouldn't use the ')' as part of the variable name
	      {  break;
	      }
              if (*ptr == '(')
	      {  isArrayMember = YES;
 	      }
              tmpArg[index++] = *ptr++; 
           } 
        }
        tmpArg[index] = '\0';
        //NXLogError("arg %d is <%s>.\n", i, tmpArg);
        strcpy(argv[i], tmpArg);
     }
     else // punt set
     {  ptr++;
     }
  }

  return argv;
}

- (const char **)nibNameArgv  {  return nibNameArgv; }
- (int)nibNameArgc  {  return nibNameArgc; }

- setNibNameArgc:(int)newArgc Argv:(char **)newArgv 
{  
   int  i;

   if (newArgc > 256)
   {  NXLogError("we only archive a max of 256 nib files, doofus...\n");
      newArgc = 256;
   }

   for (i = 0; i < nibNameArgc; i++)
   {  if (nibNameArgv[i])
      {  free(nibNameArgv[i]);
      }
   }
   nibNameArgc = newArgc;  
   for (i = 0; i < nibNameArgc; i++)
   {  nibNameArgv[i] = NXCopyStringBuffer(newArgv[i] ? newArgv[i] : "");
   }

   return self;
}

- (const char *)pathInIB { return pathInIB; }
- setPathInIB:(const char *)newPath
{
  if (pathInIB)
  {  free(pathInIB);
  }
  pathInIB = NXCopyStringBuffer(newPath ? newPath : "");

  return self;
}


/*
At this point, we want to set up a variable trace in at least one
place - this interp, and potentially another - our delegate.  Keep in
mind that the idea of the delegate is that not only does it want to be
told when a variable changes (which will be taken care of by the
WWTCLVarTrace object), it also wants to be able to apprise us of
changes it wants to make to the variable.  

Remember what's really going on (or at least what wave's gonna use
this for): 
- the delegate is actually an activeObjectProxy which has
  this instance inside it.  The real active object contains the tcl
  interp which has the final say on things, and is trying to keep in
  synch with this interp.
- a UI element might send a message to this tcl interp to change a 
  variable.  We want that change to get propagated to all UI elements
  that have expressed an interest in that variable, and we want a 
  message sent to the delegate to apprise it of the fact that the 
  variable changed.  That's the easy part; when the delegate gets sent
  such a msg, it just sends a "set var newValue" msg over the wire to 
  the real active object.  
- on the other side, we need to have set up a trace inside the real 
  active object's interp so that when that variable changes, a message
  is sent over the wire to the proxy telling it that the variable changed.
  The proxy then passes that info on to this instance by telling it to
  eval that particular message from active object.  Since this interp
  does that anyway, there's no problem.
- a difficulty arises since this is a peer-peer situation.  What happens
  when a message gets sent from a UI element to this interp.  That causes
  a trace to get triggered, which calls the procedure here.  In that procedure, 
  the appropriate WWTCLVarTrace instance sends a message to the delegate, which 
  forwards it to the active object.  Here's the problem: when the active object
  gets the message, that triggers it to send a trace message, which... you get
  the idea.  How to avoid the loop?

  Actually, looking at Chapter 32.6 of the tcl book, this looks like
  it won't be a problem.  As long as we're doing this communication inside
  the callback routine, the recursive trace won't get triggered.  I think.

  Let's see:

  hmmm....

  what I need to have is a "silentSet" command, which does what "set"
  does, except it doesn't invoke any variable traces.  Is there any
  way to do this without hacking the tcl core?  I don't think so...
  Basically, I need to reimplement Tcl_SetVar() and Tcl_SetVar2(), maybe
  calling them WWTcl_SilentSetVar() and WWTcl_SilentSetVar2()...
  This might be the simplest thing to do...  

  No.  Even simpler would be to have another set command (like set?) that
  checks the current value of the variable it's about to set.  If it's the
  same as what it's about to set it to, it doesn't do it and returns
  success.  This I can do without hacking the tcl core - cool!
*/

// declare this here just so we don't get complaints... - it's a delegate method
- traceWritesOn:(const char *)varName andUpdate:anObject  {  return self; }
- traceWritesOn2:(const char *)varName1 :(const char *)varName2 andUpdate:anObject  {  return self; }

- traceWritesOn:(const char *)varName andCall:(Tcl_VarTraceProc *)traceProcPointer usingData:(ClientData)datum
{
  id  tclTrace = [[WWTCLVarTrace alloc] initWithVarName:varName proc:traceProcPointer datum:datum delegate:delegate interp:self];

  //NXLogError("tracing writes on variable <%s> and call %p using data %p which is a %s\n", 
  //	     varName, traceProcPointer, datum, [[(id)datum class] name]);
  [traceList addObject:tclTrace];
  Tcl_TraceVar(interp, (char *)varName, TCL_TRACE_WRITES, traceProcPointer, tclTrace);
  if (delegate) {  [delegate traceWritesOn:varName andUpdate:self]; }

  return self;
}


- traceWritesOn2:(const char *)varName1 :(const char *)varName2 andCall:(Tcl_VarTraceProc *)traceProcPointer usingData:(ClientData)datum
{
  id  tclTrace = [[WWTCLVarTrace alloc] initWithVarName2:varName1 :varName2 proc:traceProcPointer datum:datum delegate:delegate interp:self];

  //NXLogError("tracing writes on variable <%s(%s)> and call %p using data %p which is a %s\n", 
  //	     varName1, varName2, traceProcPointer, datum, [[(id)datum class] name]);
  [traceList addObject:tclTrace];
  Tcl_TraceVar2(interp, (char *)varName1, (char *)varName2, TCL_TRACE_WRITES, traceProcPointer, tclTrace);
  if (delegate) {  [delegate traceWritesOn2:varName1 :varName2 andUpdate:self]; }

  return self;
}


// at this point, datum points to an instance of WWTCLVarTrace
- untraceWritesOn:(const char *)varName calling:(Tcl_VarTraceProc *)traceProcPointer usingData:(ClientData)datum
{
  int   i = 0, 
        howMany = [traceList count]; 
  BOOL  traceFound = NO;
  id    trace = nil;


  //NXLogError("untracing writes on variable <%s> calling %p using data %p which is a %s\n", 
  //	     varName, traceProcPointer, datum, [[(id)datum class] name]);
  // need to find WWTCLVarTrace that corresponds to traceProcPointer and datum
  while ((i < howMany) && (!traceFound))
  {  trace = [traceList objectAt:i];
     traceFound = [trace matchWithVarName:varName proc:traceProcPointer datum:datum delegate:delegate];
     if (!traceFound)
     { i++;
     }
  }
  if (!traceFound)
  {  NXLogError("unable to find trace for var <%s> to untrace...\n", varName);
     return nil;
  }

  [trace free];  // free'ing the trace will actually remove it from the trace list and the interp's list

  return self;
}


// at this point, datum points to an instance of WWTCLVarTrace
- _untraceWritesOn:(const char *)varName calling:(Tcl_VarTraceProc *)traceProcPointer usingData:(ClientData)datum
{
  int   i = 0, 
        howMany = [traceList count]; 
  BOOL  traceFound = NO;
  id    trace = nil;


  //NXLogError("untracing writes (but not freeing the WWTCLVarTrace object) on variable <%s> calling %p using data %p which is a %s\n", 
  //	     varName, traceProcPointer, datum, [[(id)datum class] name]);
  // need to find WWTCLVarTrace that corresponds to traceProcPointer and datum
  while ((i < howMany) && (!traceFound))
  {  trace = [traceList objectAt:i];
     traceFound = [trace matchWithVarName:varName proc:traceProcPointer datum:datum delegate:delegate];
     if (!traceFound)
     { i++;
     }
  }
  if (!traceFound)
  {  NXLogError("unable to find trace for variable <%s> calling %d using data %d which is a %s\n", 
	        varName, traceProcPointer, datum, [[(id)datum class] name]);
     return nil;
  }

  Tcl_UntraceVar(interp, (char *)varName, TCL_TRACE_WRITES, traceProcPointer, trace);
  //NXLogError("removing trace at index %d - there are %d items in the list\n", i, [traceList count]);
  [traceList removeObjectAt:i];
  //NXLogError("Now there are %d items in the list\n", [traceList count]);
  // NOTE: this routine does everything except free the trace. This routine should only be called from the trace itself

  return self;
}


// declare this here just so we don't get complaints... - it's a delegate method
- traceReadsOn:(const char *)varName andUpdate:anObject  {  return self; }

- traceReadsOn:(const char *)varName andCall:(Tcl_VarTraceProc *)traceProcPointer usingData:(ClientData)datum
{
  id  tclTrace = [[WWTCLVarTrace alloc] initWithVarName:varName proc:traceProcPointer datum:datum delegate:delegate interp:self];

  [traceList addObject:tclTrace];
  Tcl_TraceVar(interp, (char *)varName, TCL_TRACE_READS, traceProcPointer, tclTrace);
  if (delegate) {  [delegate traceReadsOn:varName andUpdate:self]; }

  return self;
}


// at this point, datum points to an instance of WWTCLVarTrace
- untraceReadsOn:(const char *)varName calling:(Tcl_VarTraceProc *)traceProcPointer usingData:(ClientData)datum
{
  int   i = 0, 
        howMany = [traceList count]; 
  BOOL  traceFound = NO;
  id    trace = nil;


  // need to find WWTCLVarTrace that corresponds to traceProcPointer and datum
  while ((i < howMany) && (!traceFound))
  {  trace = [traceList objectAt:i];
     traceFound = [trace matchWithVarName:varName proc:traceProcPointer datum:datum delegate:delegate];
     if (!traceFound)
     { i++;
     }
  }
  if (!traceFound)
  {  NXLogError("unable to find trace for var <%s> to untrace...\n", varName);
     return nil;
  }

  [trace free];  // free'ing the trace will actually remove it from the trace list and the interp's list

  return self;
}
//
- generateClosureFrom:(const char *)originalExpression andForGlobalsCall:(Tcl_VarTraceProc *)writeProc usingData:(ClientData)clientData
{
  char  **argv, *ptr, tmpArg[MAXARGLEN];  // yea, this is bad, evil, and nasty...
  int   i, j, index, len, cnt, howMany, argvLen;   // but if you name a variable longer than that, you're an ass...
  int   difference = 0;
  char  *newExpression;
  char  **newArgv, *varValue, *newPtr, *tmpPtr, *aCmd;
  BOOL  isArrayMember, foundMatch;
  List  *varNameList;
  Tcl_DString  infoLocalsCmd;
  WWTCLClosedCmd  *newClosedCmd = [[[WWTCLClosedCmd alloc] initWithInterp:self] setOriginalExpression:originalExpression];
  

  // given an expression, this routine does the following:
  // scans through, looking for all valid tcl variable names
  // for each variable foo:
  // - checks to see if there is a local variable named foo
  //   - if there is, it gets the current value of foo
  //   - if it's not, it sets up a variable trace for the 
  //     that global variable
  //     NOTE: need to only set up one trace per variable; i.e.
  //     if the same variable shows up more than once in this
  //     expression, it should still only have one trace on it.

  ptr = (char *)originalExpression;
  cnt = 0; 
  howMany = strlen(originalExpression);
  // looking for '$', 'set '

  // scan the first time, counting how many args
  // scan the next time, counting how long each is
  // scan the final time, copying the strings in
  argvLen = 0;
  while ((ptr-originalExpression) < howMany)
  {
     if (*ptr == '$')
     {  argvLen++; ptr++; 
        isArrayMember = NO;
        // it might have a { right after $
        if (*ptr == '{')
	{  ptr++; while ((*ptr)  && (*ptr != '}')) { ptr++; }
	}
        else // could be in an expression, which has lots of operators that can be butted up against each other...
        {  while ((*ptr)  && !isspace(*ptr) 
                          && (*ptr != '$') 
                          && (*ptr != '+') && (*ptr != '-') && (*ptr != '*') && (*ptr != '/') && (*ptr != '%') 
                          && (*ptr != '!') && (*ptr != '~') && (*ptr != '<') && (*ptr != '>') && (*ptr != '=') 
                          && (*ptr != '&') && (*ptr != '^') && (*ptr != '|') && (*ptr != '?') && (*ptr != ':')
                          && (*ptr != '[') && (*ptr != ']') && (*ptr != '}') && (*ptr != '"'))
           {  if (*ptr == ')')
              {  if(!isArrayMember) // shouldn't use the ')' as part of the variable name
	         {  break;
	         }
                 else // should use the ')' as part of the variable name, and get out
                 {  ptr++;
	            break;
	         }
              }             
              if (*ptr != '(')
	      {  isArrayMember = YES;
 	      }
              ptr++; 
           } 
        }
     }
     else
     {  ptr++;
     }
  }

  if (argvLen)
  {  if (argvLen == 1)
     {  //NXLogError("I think there is one variable in the command.\n");
     }
     else
     {  //NXLogError("I think there are %d variables in the command.\n", argvLen);
     }
  }
  else
  {  //NXLogError("I don't think there are any variables in the command. - why do you have it in an EveCmd?\n");
     // make a copy of the original expression and return that.
     return newClosedCmd;
  }

  Tcl_DStringInit(&infoLocalsCmd);
  Tcl_DStringAppend(&infoLocalsCmd, "info locals ", -1);
  varNameList = [[List alloc] initCount:argvLen];
  argv = (char **)malloc(sizeof(char *) * (argvLen+1));
  argv[argvLen] = NULL;  // mark the end of it

  ptr = (char *)originalExpression;
  i = -1;
  len = 0;
  while ((ptr-originalExpression) < howMany)
  {  if (*ptr && (*ptr == '$'))
     {  i++; ptr++; len = 0;
        isArrayMember = NO;
        // it might have a { right after $
        if (*ptr && (*ptr == '{'))
	{  ptr++; while ((*ptr)  && (*ptr != '}')) { ptr++; len++; }
	}
        else // could be in an expression, which has lots of operators that can be butted up against each other...
        {  while ((*ptr)  && !isspace(*ptr) 
                          && (*ptr != '$') 
                          && (*ptr != '+') && (*ptr != '-') && (*ptr != '*') && (*ptr != '/') && (*ptr != '%') 
                          && (*ptr != '!') && (*ptr != '~') && (*ptr != '<') && (*ptr != '>') && (*ptr != '=') 
                          && (*ptr != '&') && (*ptr != '^') && (*ptr != '|') && (*ptr != '?') && (*ptr != ':')
                          && (*ptr != '[') && (*ptr != ']') && (*ptr != '}') && (*ptr != '"'))
           {  if (*ptr == ')')
              {  if(!isArrayMember) // shouldn't use the ')' as part of the variable name
	         {  break;
	         }
                 else // should use the ')' as part of the variable name, and get out
                 {  ptr++; len++;
	            break;
	         }
              }             
              if (*ptr == '(')
	      {  isArrayMember = YES;
 	      }
              ptr++; len++;
           } 
        }
        len++;
        //NXLogError("arg %d has %d characters (including NULL) in it.\n", i, len);
        argv[i] = (char *)malloc(len);
     }
     else
     {  ptr++;
     }
  }

  ptr = (char *)originalExpression;
  i = -1;
  index = 0;
  while ((ptr-originalExpression) < howMany)
  {  if (*ptr && (*ptr == '$'))
     {  i++; ptr++; index = 0;
        isArrayMember = NO;
        // it might have a { right after $
        if (*ptr && (*ptr == '{'))
	{  ptr++; while ((*ptr)  && (*ptr != '}')) { tmpArg[index++] = *ptr++; }
	}
        else // could be in an expression, which has lots of operators that can be butted up against each other...
        {  while ((*ptr)  && !isspace(*ptr) 
                          && (*ptr != '$') 
                          && (*ptr != '+') && (*ptr != '-') && (*ptr != '*') && (*ptr != '/') && (*ptr != '%') 
                          && (*ptr != '!') && (*ptr != '~') && (*ptr != '<') && (*ptr != '>') && (*ptr != '=') 
                          && (*ptr != '&') && (*ptr != '^') && (*ptr != '|') && (*ptr != '?') && (*ptr != ':')
                          && (*ptr != '[') && (*ptr != ']') && (*ptr != '}') && (*ptr != '"'))
           {  if (*ptr == ')')
              {  if(!isArrayMember) // shouldn't use the ')' as part of the variable name
	         {  break;
	         }
                 else // should use the ')' as part of the variable name, and get out
                 {  tmpArg[index++] = *ptr++; 
	            break;
	         }
              }             
              if (*ptr == '(')
	      {  isArrayMember = YES;
 	      }
              tmpArg[index++] = *ptr++; 
           } 
        }
        tmpArg[index] = '\0';
        //NXLogError("arg %d is <%s>.\n", i, tmpArg);
        strcpy(argv[i], tmpArg);
     }
     else // look for set
     {  ptr++;
     }
  }

  // at this point, we know we have argvLen variables in the expression.
  // we now need to determine which are local variables and which are global.
  // For the local variables, we want to look up their value and substitute
  // that into the newExpression for each occurrence of the local variable.
  // For the global variables, we want to set up a trace using the passed 
  // in info.
  // get a list of all the local variables

  //if (Tcl_Eval(interp, "info locals") !=  TCL_OK)
  //{  //NXLogError("yikes! unable to get info on local variables...");
  //   // need to free stuff...
  //   free(argv);
  //   return nil;
  //}
  //Tcl_SplitList(interp, interp->result, &localVarNameC, &localVarNameV);
  //Tcl_ResetResult(interp); 

  newArgv = (char **)malloc(sizeof(char *) * (argvLen+1));
  newArgv[argvLen] = NULL;  // mark the end of it
  difference = 0;

  for (i = 0; i < argvLen; i++)
  {  // is this a local variable?
     varValue = Tcl_GetVar(interp, argv[i], 0);
     if (!varValue)
     {  // got a NULL back - it's an unknown variable
        sprintf(errBuf, "unknown variable %s", argv[i]);
        Tcl_AppendResult(interp, errBuf, (char *)NULL);
        return NULL;
     }
     else
     {  // make a copy; we'll need this either way
        newArgv[i] = NXCopyStringBuffer(argv[i]);

        // it's either global or local 
        //   - if local, get value, replace into newExpression
        //   - if global, add to list
        Tcl_DStringInit(&infoLocalsCmd);
        Tcl_DStringAppend(&infoLocalsCmd, "info locals ", -1);
        aCmd = Tcl_DStringAppend(&infoLocalsCmd, argv[i], -1);
        if (Tcl_Eval(interp, aCmd) != TCL_OK)
        {  //NXLogError("Yikes! problem generating closure: (evaluating <%s> yielded <%s>)\n", 
	   //      aCmd, interp->result);
           Tcl_DStringFree(&infoLocalsCmd);
           // wave NOTE: need to unroll through the newArgv's that have been malloc'ed thus far and free...

           return NULL;
        }

        // check and see if the return string is just blanks or not...
        if (notJustBlanks(interp->result))
        {  // it's a valid, local variable
           difference += strlen(newArgv[i]) - strlen(argv[i]);
        }
        else  // must be a valid global variable
        {  // since it's global - set up trace
           // first we have to see if it appears in the list of variables
           // for this command that we've already set up traces on - no need to 
           // trace it twice for the same command...
           foundMatch = NO;  j = 0;
           while (!foundMatch && j < [varNameList count])
           {  if ([[varNameList objectAt:j] sameAs:argv[i]])
              {  foundMatch = YES;
              }
              j++;
           }
           if (!foundMatch)
           {  [self traceWritesOn:argv[i] andCall:writeProc usingData:clientData];
              [newClosedCmd addTraceFor:argv[i] calling:writeProc using:clientData];
              [varNameList addObject:[[WWSimpleString alloc] initWithStr:argv[i]]];
           }
           else
           {  //NXLogError("already set up trace on <%s>\n", argv[i]);
           }
        }
        Tcl_ResetResult(interp);
     } 
  }
  Tcl_DStringFree(&infoLocalsCmd);

  // we need to know how much longer (if any) the new expression is than the old one.
  // the easiest way to do that is to compare the total length of the old variables 
  // with the new ones.  Have we kept track so far?

  // okay, let's go through the expression one more time, copying the 
  newExpression = (char *)malloc(1 + strlen(originalExpression) + difference);
  newPtr = newExpression;
  ptr = (char *)originalExpression;
  i = 0;
  while ((ptr-originalExpression) < howMany)
  {  if (*ptr == '$')
     {  // if this one is a local variable, we need to punt the $ and copy in the actual value
        isArrayMember = NO;
        if (!strcmp(argv[i], newArgv[i]))  // they're the same; it's global!
        {  *newPtr++ = *ptr++; // toss in the $
           // it might have a { right after $
           if (*ptr == '{')
           {  *newPtr++ = *ptr++; while ((*ptr)  && (*ptr != '}')) { *newPtr++ = *ptr++; }
   	   }
           else // could be in an expression, which has lots of operators that can be butted up against each other...
           {  while ((*ptr)  && !isspace(*ptr) 
                             && (*ptr != '$') 
                             && (*ptr != '+') && (*ptr != '-') && (*ptr != '*') && (*ptr != '/') && (*ptr != '%') 
                             && (*ptr != '!') && (*ptr != '~') && (*ptr != '<') && (*ptr != '>') && (*ptr != '=') 
                             && (*ptr != '&') && (*ptr != '^') && (*ptr != '|') && (*ptr != '?') && (*ptr != ':')
                             && (*ptr != '[') && (*ptr != ']') && (*ptr != '}') && (*ptr != '"'))
             {   if (*ptr == ')')
                 {   if(!isArrayMember) // shouldn't use the ')' as part of the variable name
	            {  break;
	            }
                    else // should use the ')' as part of the variable name, and get out
                    {  *newPtr++ = *ptr++; 
	               break;
	            }
                 }             
                 if (*ptr == '(')
  	         {  isArrayMember = YES;
 	         }
                 *newPtr++ = *ptr++; 
              } 
           }
           i++;
        }
        else // it's local; lose the $ and stick in the value... 
        {  // first copy in the current value into the newExpression
           tmpPtr = newArgv[i];
           while (*tmpPtr) { *newPtr++ = *tmpPtr++; }
           // okay, now we have to get ptr to point past this variable
           ptr++;  // get past the $
           // it might have a { right after $
           if (*ptr == '{')
           {  ptr++;  while ((*ptr)  && (*ptr != '}')) { ptr++; }
   	   }
           else // could be in an expression, which has lots of operators that can be butted up against each other...
           {  while ((*ptr)  && !isspace(*ptr) 
                             && (*ptr != '$') 
                             && (*ptr != '+') && (*ptr != '-') && (*ptr != '*') && (*ptr != '/') && (*ptr != '%') 
                             && (*ptr != '!') && (*ptr != '~') && (*ptr != '<') && (*ptr != '>') && (*ptr != '=') 
                             && (*ptr != '&') && (*ptr != '^') && (*ptr != '|') && (*ptr != '?') && (*ptr != ':')
                             && (*ptr != '[') && (*ptr != ']') && (*ptr != '}') && (*ptr != '"'))
              {  if (*ptr == ')')
                 {  if(!isArrayMember) // shouldn't use the ')' as part of the variable name
	            {  break;
	            }
                    else // should use the ')' as part of the variable name, and get out
                    {  tmpArg[index++] = *ptr++; 
	               break;
	            }
                 }             
                 if (*ptr == '(')
  	         {  isArrayMember = YES;
 	         }
                 tmpArg[index++] = *ptr++; 
              } 
           }
           i++;
	}
     }
     else // punt set
     {  *newPtr++ = *ptr++;
     }
  }
  *newPtr = '\0';

  // okay, free stuff!
  for (i = 0; i < argvLen; i++)
  {  free(argv[i]);
  }
  free(argv);
  for (i = 0; i < argvLen; i++)
  {  free(newArgv[i]);
  }
  free(newArgv);

  [newClosedCmd setClosedCmd:newExpression];
  free(newExpression);

  return newClosedCmd;
}

- linkVarNamed:(const char *)varName toPointer:(char *)cPointer withFlags:(int)flags
{
  Tcl_LinkVar(interp, (char *)varName, cPointer, flags);
  return self;
}


- unlinkVarNamed:(const char *)varName
{
  Tcl_UnlinkVar(interp, (char *)varName);
  return self;
}


// window delegate methods...
- windowWillClose:sender
{
  Tcl_HashEntry  *entryPtr;
  int            i, cnt, varCnt, k;
  id             theList, theObject;
  BOOL           found = NO, gotIt;
  char           **varV;
  const char     *anExpression, *aTclVar;

  
  if (errorWindow && (sender == errorWindow))
  {  return [self closeErrorWindow:sender];
  }
  entryPtr = Tcl_FindHashEntry(&uiHashTableByWindow, (char *)sender); 
  if (entryPtr)
  {  theList = (List *)Tcl_GetHashValue(entryPtr);
     cnt = [theList count];
     for (i = 0; i < cnt; i++)
     {  theObject = [theList objectAt:i];
        gotIt = NO;
        if ([theObject respondsTo:@selector(tclVar)])
	{  aTclVar = [theObject tclVar];
	   if (notJustBlanks(aTclVar))
   	   {  [self untraceWritesOn:(const char *)aTclVar calling:(Tcl_VarTraceProc *)WriteProcForTclVar usingData:(ClientData)theObject];
              gotIt = YES;
	   }
        }
	if (!gotIt && [theObject respondsTo:@selector(tclExpression)])
	{  anExpression = [theObject tclExpression];
	   if (notJustBlanks(anExpression))
	   {  varV = [self argvFromCmd:anExpression];
	      varCnt = 0;
	      if (varV)
	      {  while (varV[varCnt])
		 {  varCnt++;
		 }
	      }
	      for (k = 0; k < varCnt; k++)
	      {  [self untraceWritesOn:varV[k] calling:(Tcl_VarTraceProc *)WriteProcForTclExpression usingData:(ClientData)theObject];
		 free(varV[k]);
	      }
   	      free(varV);
	   }
	}
     }
     [theList free];
     Tcl_DeleteHashEntry(entryPtr);
  }
  
  // now we need to find "sender" in the controlPanelList and delete it, so we don't try to close it when we get free'ed...
  i = 0; cnt = [controlPanelList count];
  while (!found && (i < cnt))
  {  if ([controlPanelList objectAt:i] == sender)
     {  [controlPanelList removeObjectAt:i];
        found = YES;
     }
     i++;
  }

  return self;
}

- (const char *)getInspectorClassName { return "WWTCLInterpIBInspector"; }

- (NXImage *)getIBImage
{
//  This returns the NXImage instance that will be displayed in IB's File Viewer
//  when this class is instantiated.  (Assumes image is in palette nib.  There are
//  alternatives. This should have been initFromSection but something with 
//  IB loading palette images is misbehaving...).
	
  id	bundle;
  char 	path [MAXPATHLEN + 1];


  bundle = [NXBundle bundleForClass: [WWTCLInterp class]];
  if (!bundle) 
  {  NXLogError("No bundle for Class %s.\n", [WWTCLInterp name]);
     return nil;
  }
			
  if (![bundle getPath: path forResource: "WWTCLInterp" ofType: "tiff"])
  {  NXLogError("No path for WWTCLInterpPalette.tiff.\n");
     return nil;
  }

  return [[NXImage alloc] initFromFile:path];
}




- writeState:(NXStream *)stream 
{
   int          i, j, argc, argc2, ret, cnt;
   char         **argv, **argv2, *varNameList, *procNameList, *aCmd, *varValue, *arrayElementList, *arrayName;
   Tcl_DString  setCmd, infoCmd, procCmd, arrayElementCmd;


   // at this point, we need to suck out the names and values of all
   // the global variables in the tcl interp.  Then we need to get 
   // the names, args, and bodies of all the procs that have been 
   // defined.  We write these out, and then when we do a read:, 
   // we shove those back into the interp.

   // okay, so first we send "info globals" to the interp.  
   // We'll get back a string which we'll do a Tcl_SplitList on.
   // we'll write out the argc of that list to the stream.
   // for each element in the string, we'll stick "set " in front of it 
   // and evaluate it.  We'll then save the string "set <variable> <value>"
   // out to the stream.

   // we'll then send "info procs" to the interp.
   // We'll get back a string which we'll do a Tcl_SplitList on.
   // we'll write out the argc of that list to the stream.
   // for each element in the string, we'll call that element "procName"
   // for each procName, we'll then send "info args <procName>" and call the
   // result "procArgs",  we'll then send "info body <procName>" and call the
   // the result "procBody",
   // we'll then sprintf into a string variable: 
   // sprintf(procDef, "proc %s {%s} {%s}", procName, procArgs, procBody);
   // and save "procDef" out to the stream

   ret = Tcl_GlobalEval(interp, "info globals"); 
   if (ret != TCL_OK)
   {  NXLogError("Yikes! problem in WWTCLInterp's write: (evaluating <info globals> yielded <%s>)\n", 
	         interp->result);
      [self showError:interp->result];
      varNameList = NXCopyStringBuffer("");
   }
   else
   {  varNameList = NXCopyStringBuffer(interp->result); 
   }
   Tcl_ResetResult(interp); 

   argc = 0;
   argv = NULL;
   Tcl_SplitList(interp, varNameList, &argc, &argv);
   Tcl_ResetResult(interp); 

   cnt = 0;
   for (i = 0; i < argc; i++)
   {  if (strcmp(argv[i], "env")) // archive everything *but* env
      {  varValue = Tcl_GetVar(interp, argv[i], TCL_GLOBAL_ONLY);
         if (varValue)
         {  cnt++; 
         }
         else
         {  Tcl_DStringInit(&arrayElementCmd);
            Tcl_DStringAppend(&arrayElementCmd, "array names ", -1);
            aCmd = Tcl_DStringAppend(&arrayElementCmd, argv[i], -1);
            ret = Tcl_GlobalEval(interp, aCmd);
            if (ret != TCL_OK)
            {  NXLogError("Yikes! problem in WWTCLInterp's write: (evaluating <%s> yielded <%s>)\n", 
   	               aCmd, interp->result);
               [self showError:interp->result];
               arrayElementList = NXCopyStringBuffer("");
            }
            else
            {  arrayElementList = NXCopyStringBuffer(interp->result); 
            }
            Tcl_ResetResult(interp); 
            argc2 = 0;
            argv2 = NULL;
            Tcl_SplitList(interp, arrayElementList, &argc2, &argv2);
            for (j = 0; j < argc2; j++)
            {  cnt++;
            }
            Tcl_ResetResult(interp); 
            if (argv2) {  free(argv2); }
            free(arrayElementList);  // it was malloc'ed either way
         }
      }
   }
   free(varNameList);
   varNameList = NULL;

   // we now check to see if these strings are going to put us over the limit
   // on our tmp string array.  If they will, realloc.
   while ((tmpStringIndex + cnt) > tmpStringc)
   {  tmpStringc *= 2;
      tmpStringv = (char **)NXZoneRealloc([self zone], tmpStringv, (sizeof(char *) * tmpStringc));
   }

   NXPrintf(stream, "# Writing out all the current values of the global variables:\n");
   for (i = 0; i < argc; i++)
   {  if (strcmp(argv[i], "env")) // archive everything *but* env
      {  Tcl_DStringInit(&setCmd);
         Tcl_DStringAppend(&setCmd, "set ", -1);
         Tcl_DStringAppend(&setCmd, argv[i], -1);
         varValue = Tcl_GetVar(interp, argv[i], TCL_GLOBAL_ONLY);
         if (varValue)
         {  Tcl_DStringAppend(&setCmd, " {", -1);
            Tcl_DStringAppend(&setCmd, varValue, -1);
            aCmd = tmpStringv[tmpStringIndex++] = NXCopyStringBuffer(Tcl_DStringAppend(&setCmd, "}", -1));
            NXPrintf(stream, "%s;\n", aCmd);
            Tcl_DStringFree(&setCmd);
         }
         else
         {  // if we failed to get a value for the var, it must be an array.
            // need to find out how many elements there are and save out each one.
            // eval "array names <var>", do a split list on it, etc.
   
            Tcl_DStringInit(&arrayElementCmd);
            Tcl_DStringAppend(&arrayElementCmd, "array names ", -1);
            aCmd = Tcl_DStringAppend(&arrayElementCmd, argv[i], -1);
            ret = Tcl_GlobalEval(interp, aCmd);
            if (ret != TCL_OK)
            {  NXLogError("Yikes! problem in WWTCLInterp's write: (evaluating <%s> yielded <%s>)\n", 
   	               aCmd, interp->result);
               [self showError:interp->result];
               arrayElementList = NXCopyStringBuffer(""); 
            }
            else
            {   arrayElementList = NXCopyStringBuffer(interp->result); 
            }
            Tcl_ResetResult(interp); 
   
            arrayName = NXCopyStringBuffer(Tcl_DStringAppend(&setCmd, "", 0));
            argc2 = 0;
            argv2 = NULL;
            Tcl_SplitList(interp, arrayElementList, &argc2, &argv2);
            for (j = 0; j < argc2; j++)
            {  // need to join the current 
               Tcl_DStringInit(&setCmd);
               Tcl_DStringAppend(&setCmd, arrayName, -1);
               Tcl_DStringAppend(&setCmd, "(", -1);
               Tcl_DStringAppend(&setCmd, argv2[j], -1);
               aCmd = Tcl_DStringAppend(&setCmd, ")", -1);
               ret = Tcl_GlobalEval(interp, aCmd);
               if (ret != TCL_OK)
               {  NXLogError("Yikes! problem in WWTCLInterp's write: (evaluating <%s> yielded <%s>)\n", 
   	                  aCmd, interp->result);
                  [self showError:interp->result];
                  aCmd = tmpStringv[tmpStringIndex++] = NXCopyStringBuffer("BAD VALUE");
               }
               else
               {  Tcl_DStringAppend(&setCmd, " {", -1);
                  Tcl_DStringAppend(&setCmd, interp->result, -1);
                  aCmd = tmpStringv[tmpStringIndex++] = NXCopyStringBuffer(Tcl_DStringAppend(&setCmd, "}", -1));
               }
               Tcl_ResetResult(interp); 
   
               NXPrintf(stream, "%s;\n", aCmd);
               Tcl_DStringFree(&setCmd);
            }
            if (arrayName)  {  free(arrayName); }
         }
      }
   }
   if (argv) {  free(argv); }
   NXPrintf(stream, "# end of globals\n");

   NXPrintf(stream, "# Writing out all the current definitions of all the non built-in procs:\n");
   ret = Tcl_GlobalEval(interp, "info procs"); 
   if (ret != TCL_OK)
   {  NXLogError("Yikes! problem in WWTCLInterp's write: (evaluating <info procs> yielded <%s>)\n", 
	         interp->result);
      [self showError:interp->result];
   }
   procNameList = NXCopyStringBuffer(interp->result); 
   Tcl_ResetResult(interp); 

   Tcl_SplitList(interp, procNameList, &argc, &argv);

   // we now check to see if these strings are going to put us over the limit
   // on our tmp string array.  If they will, realloc.
   while ((tmpStringIndex + cnt) > tmpStringc)
   {  tmpStringc *= 2;
      tmpStringv = (char **)NXZoneRealloc([self zone], tmpStringv, (sizeof(char *) * tmpStringc));
   }

   for (i = 0; i < argc; i++)
   {  Tcl_DStringInit(&procCmd);
      Tcl_DStringAppend(&procCmd, "proc ", -1);
      Tcl_DStringAppend(&procCmd, argv[i], -1);
      Tcl_DStringAppend(&procCmd, " { ", -1);

      Tcl_DStringInit(&infoCmd);
      Tcl_DStringAppend(&infoCmd, "info args ", -1);
      aCmd = Tcl_DStringAppend(&infoCmd, argv[i], -1);
      ret = Tcl_GlobalEval(interp, aCmd);
      if (ret != TCL_OK)
      {  NXLogError("Yikes! problem in WWTCLInterp's write: (evaluating <%s> yielded <%s>)\n", 
	            aCmd, interp->result);
         [self showError:interp->result];
         Tcl_ResetResult(interp); 
         aCmd = tmpStringv[tmpStringIndex++] = NXCopyStringBuffer(Tcl_DStringAppend(&procCmd, 
                                                                                    "} {echo {this proc had bad args\\n} }", -1));
      }
      else
      {  Tcl_DStringAppend(&procCmd, interp->result, -1);
         Tcl_DStringAppend(&procCmd, " } { ", -1);
         Tcl_ResetResult(interp); 
         Tcl_DStringFree(&infoCmd);

         Tcl_DStringInit(&infoCmd);
         Tcl_DStringAppend(&infoCmd, "info body ", -1);
         aCmd = Tcl_DStringAppend(&infoCmd, argv[i], -1);
         ret = Tcl_GlobalEval(interp, aCmd);
         if (ret != TCL_OK)
         {  NXLogError("Yikes! problem in WWTCLInterp's write: (evaluating <%s> yielded <%s>)\n", 
   	               aCmd, interp->result);
            [self showError:interp->result];
            aCmd = tmpStringv[tmpStringIndex++] = NXCopyStringBuffer(Tcl_DStringAppend(&procCmd, 
                                                                                    "echo {this proc had a bad body\\n} }", -1));
         }
         else
         {  Tcl_DStringAppend(&procCmd, interp->result, -1);
            Tcl_DStringFree(&infoCmd);

            aCmd = tmpStringv[tmpStringIndex++] = NXCopyStringBuffer(Tcl_DStringAppend(&procCmd, " }", -1));
         }
         Tcl_ResetResult(interp); 
      }
      //NXLogError("writing out: <%s> at address %d\n", aCmd, aCmd);
      NXPrintf(stream, "%s;\n", aCmd);
      Tcl_DStringFree(&procCmd);

   }
   NXPrintf(stream, "# end of procs\n");
   if (argv) {  free(argv); }
   if (procNameList) {  free(procNameList); }

   return self;
}




///////////////////////////////////////////////////////////////////////////////

#define typeVectorVersion123 "*i"
#define typeValuesVersion123 &pathInIB, &nibNameArgc

#define typeVector "i"
#define typeValues &nibNameArgc

- read:(NXTypedStream*)stream 
{
   int version, i;
   [super read:stream];


NX_DURING
   nibNameArgv = (char **)malloc(256 * sizeof(char *));

   version = NXTypedStreamClassVersion(stream, "WWTCLInterp");
   if (version == 0)
   {  pathInIB = NULL;
      nibNameArgc = 0;
      delegate = nil;
   }
   if (version == 1)
   {  NXReadTypes(stream, typeVectorVersion123, typeValuesVersion123);
      if (nibNameArgc > 256)
      {  NXLogError("we only archive a max of 256 nib files, doofus...\n");
      }
      for (i = 0; i < nibNameArgc; i++)
      {  NXReadType(stream, "*", &(nibNameArgv[i]));
         //NXLogError("\t<%s>\n", nibNameArgv[i]); 
      }
      delegate = nil;
      setc = 0;
      setv = NULL;
      procc = 0;
      procv = NULL;
      // if we archived pathInIB, we'll just throw it away, cause it might be wrong...
      if (pathInIB) {  NXZoneFree([self zone], pathInIB); }
   }
   if (version == 2)
   {  NXReadTypes(stream, typeVectorVersion123, typeValuesVersion123);
      if (nibNameArgc > 256)
      {  NXLogError("we only archive a max of 256 nib files, doofus...\n");
      }
      for (i = 0; i < nibNameArgc; i++)
      {  NXReadType(stream, "*", &(nibNameArgv[i]));
         //NXLogError("\t<%s>\n", nibNameArgv[i]); 
      }
      delegate = NXReadObject(stream);
      setc = 0;
      setv = NULL;
      procc = 0;
      procv = NULL;
      // if we archived pathInIB, we'll just throw it away, cause it might be wrong...
      if (pathInIB) {  NXZoneFree([self zone], pathInIB); }
   }
   if (version == 3)
   {  NXReadTypes(stream, typeVectorVersion123, typeValuesVersion123);
      if (nibNameArgc > 256)
      {  NXLogError("we only archive a max of 256 nib files, doofus...\n");
      }
      for (i = 0; i < nibNameArgc; i++)
      {  NXReadType(stream, "*", &(nibNameArgv[i]));
         //NXLogError("\t<%s>\n", nibNameArgv[i]); 
      }
      delegate = NXReadObject(stream);

      // how many set's do we have to do?
      NXReadType(stream, "i", &setc);
      //NXLogError("read in: <%d>\n", setc);
      setv = (char **)malloc((sizeof(char *) * setc));
      for (i = 0; i < setc; i++)
      {  NXReadTypes(stream, "*", &(setv[i]));
         //NXLogError("read in: <%s>\n", setv[i]);
      }

      // how many proc's must we define?
      NXReadType(stream, "i", &procc);
      //NXLogError("read in: <%d>\n", procc);
      procv = (char **)malloc((sizeof(char *) * procc));
      for (i = 0; i < procc; i++)
      {  NXReadTypes(stream, "*", &(procv[i]));
         //NXLogError("read in: <%s>\n", procv[i]);
      }
      // if we archived pathInIB, we'll just throw it away, cause it might be wrong...
      if (pathInIB) {  free(pathInIB); }
   }
   if (version == 4)
   {  NXReadTypes(stream, typeVector, typeValues);
      if (nibNameArgc > 256)
      {  NXLogError("we only archive a max of 256 nib files, doofus...\n");
      }
      for (i = 0; i < nibNameArgc; i++)
      {  NXReadType(stream, "*", &(nibNameArgv[i]));
         //NXLogError("\t<%s>\n", nibNameArgv[i]); 
      }
      delegate = NXReadObject(stream);

      // how many set's do we have to do?
      NXReadType(stream, "i", &setc);
      //NXLogError("read in: <%d>\n", setc);
      setv = (char **)malloc(sizeof(char *) * setc);
      for (i = 0; i < setc; i++)
      {  NXReadTypes(stream, "*", &(setv[i]));
         //NXLogError("read in: <%s>\n", setv[i]);
      }

      // how many proc's must we define?
      NXReadType(stream, "i", &procc);
      //NXLogError("read in: <%d>\n", procc);
      procv = (char **)malloc((sizeof(char *) * procc));
      for (i = 0; i < procc; i++)
      {  NXReadTypes(stream, "*", &(procv[i]));
         //NXLogError("read in: <%s>\n", procv[i]);
      }
   }
NX_HANDLER
   NXLogError("in read: %s, exception [%d] raised.\n", 
                 [[self class] name], NXLocalHandler.code);
   return nil;
NX_ENDHANDLER
   return self;
}


- write:(NXTypedStream *)stream 
{
   int          i, j, argc, argc2, ret, cnt;
   char         **argv, **argv2, *varNameList, *procNameList, *aCmd, *varValue, *arrayElementList, *arrayName;
   Tcl_DString  setCmd, infoCmd, procCmd, arrayElementCmd;


   [super write:stream];

   NXWriteTypes(stream, typeVector, typeValues);
   for (i = 0; i < nibNameArgc; i++)
   {  //NXLogError("\t<%s>\n", nibNameArgv[i]); 
      NXWriteType(stream, "*", &(nibNameArgv[i]));
   }
   NXWriteObjectReference(stream, delegate);

   // at this point, we need to suck out the names and values of all
   // the global variables in the tcl interp.  Then we need to get 
   // the names, args, and bodies of all the procs that have been 
   // defined.  We write these out, and then when we do a read:, 
   // we shove those back into the interp.

   // okay, so first we send "info globals" to the interp.  
   // We'll get back a string which we'll do a Tcl_SplitList on.
   // we'll write out the argc of that list to the stream.
   // for each element in the string, we'll stick "set " in front of it 
   // and evaluate it.  We'll then save the string "set <variable> <value>"
   // out to the stream.

   // we'll then send "info procs" to the interp.
   // We'll get back a string which we'll do a Tcl_SplitList on.
   // we'll write out the argc of that list to the stream.
   // for each element in the string, we'll call that element "procName"
   // for each procName, we'll then send "info args <procName>" and call the
   // result "procArgs",  we'll then send "info body <procName>" and call the
   // the result "procBody",
   // we'll then sprintf into a string variable: 
   // sprintf(procDef, "proc %s {%s} {%s}", procName, procArgs, procBody);
   // and save "procDef" out to the stream

   ret = Tcl_GlobalEval(interp, "info globals"); 
   if (ret != TCL_OK)
   {  NXLogError("Yikes! problem in WWTCLInterp's write: (evaluating <info globals> yielded <%s>)\n", 
	         interp->result);
      [self showError:interp->result];
      varNameList = NXCopyStringBuffer("");
   }
   else
   {  varNameList = NXCopyStringBuffer(interp->result); 
   }
   Tcl_ResetResult(interp); 

   argc = 0;
   argv = NULL;
   Tcl_SplitList(interp, varNameList, &argc, &argv);
   Tcl_ResetResult(interp); 

   cnt = 0;
   for (i = 0; i < argc; i++)
   {  if (strcmp(argv[i], "env")) // archive everything *but* env
      {  varValue = Tcl_GetVar(interp, argv[i], TCL_GLOBAL_ONLY);
         if (varValue)
         {  cnt++; 
         }
         else
         {  Tcl_DStringInit(&arrayElementCmd);
            Tcl_DStringAppend(&arrayElementCmd, "array names ", -1);
            aCmd = Tcl_DStringAppend(&arrayElementCmd, argv[i], -1);
            ret = Tcl_GlobalEval(interp, aCmd);
            if (ret != TCL_OK)
            {  NXLogError("Yikes! problem in WWTCLInterp's write: (evaluating <%s> yielded <%s>)\n", 
   	               aCmd, interp->result);
               [self showError:interp->result];
               arrayElementList = NXCopyStringBuffer("");
            }
            else
            {  arrayElementList = NXCopyStringBuffer(interp->result); 
            }
            Tcl_ResetResult(interp); 
            argc2 = 0;
            argv2 = NULL;
            Tcl_SplitList(interp, arrayElementList, &argc2, &argv2);
            for (j = 0; j < argc2; j++)
            {  cnt++;
            }
            Tcl_ResetResult(interp); 
            if (argv2) {  free(argv2); }
            free(arrayElementList);  // it was malloc'ed either way
         }
      }
   }
   free(varNameList);
   varNameList = NULL;

   //NXLogError("writing out: <%d> (that's how many set commands we're now going to write out)\n", cnt);
   NXWriteType(stream, "i", &cnt);

   // we now check to see if these strings are going to put us over the limit
   // on our tmp string array.  If they will, realloc.
   while ((tmpStringIndex + cnt) > tmpStringc)
   {  tmpStringc *= 2;
      tmpStringv = (char **)NXZoneRealloc([self zone], tmpStringv, (sizeof(char *) * tmpStringc));
   }

   for (i = 0; i < argc; i++)
   {  if (strcmp(argv[i], "env")) // archive everything *but* env
      {  Tcl_DStringInit(&setCmd);
         Tcl_DStringAppend(&setCmd, "set ", -1);
         Tcl_DStringAppend(&setCmd, argv[i], -1);
         varValue = Tcl_GetVar(interp, argv[i], TCL_GLOBAL_ONLY);
         if (varValue)
         {  Tcl_DStringAppend(&setCmd, " {", -1);
            Tcl_DStringAppend(&setCmd, varValue, -1);
            aCmd = tmpStringv[tmpStringIndex++] = NXCopyStringBuffer(Tcl_DStringAppend(&setCmd, "}", -1));
            //NXLogError("writing out: <%s> at address %d\n", aCmd, aCmd);
            NXWriteType(stream, "*", &aCmd);
            Tcl_DStringFree(&setCmd);
         }
         else
         {  // if we failed to get a value for the var, it must be an array.
            // need to find out how many elements there are and save out each one.
            // eval "array names <var>", do a split list on it, etc.
   
            Tcl_DStringInit(&arrayElementCmd);
            Tcl_DStringAppend(&arrayElementCmd, "array names ", -1);
            aCmd = Tcl_DStringAppend(&arrayElementCmd, argv[i], -1);
            ret = Tcl_GlobalEval(interp, aCmd);
            if (ret != TCL_OK)
            {  NXLogError("Yikes! problem in WWTCLInterp's write: (evaluating <%s> yielded <%s>)\n", 
   	               aCmd, interp->result);
               [self showError:interp->result];
               arrayElementList = NXCopyStringBuffer(""); 
            }
            else
            {   arrayElementList = NXCopyStringBuffer(interp->result); 
            }
            Tcl_ResetResult(interp); 
   
            arrayName = NXCopyStringBuffer(Tcl_DStringAppend(&setCmd, "", 0));
            argc2 = 0;
            argv2 = NULL;
            Tcl_SplitList(interp, arrayElementList, &argc2, &argv2);
            for (j = 0; j < argc2; j++)
            {  // need to join the current 
               Tcl_DStringInit(&setCmd);
               Tcl_DStringAppend(&setCmd, arrayName, -1);
               Tcl_DStringAppend(&setCmd, "(", -1);
               Tcl_DStringAppend(&setCmd, argv2[j], -1);
               aCmd = Tcl_DStringAppend(&setCmd, ")", -1);
               ret = Tcl_GlobalEval(interp, aCmd);
               if (ret != TCL_OK)
               {  NXLogError("Yikes! problem in WWTCLInterp's write: (evaluating <%s> yielded <%s>)\n", 
   	                  aCmd, interp->result);
                  [self showError:interp->result];
                  aCmd = tmpStringv[tmpStringIndex++] = NXCopyStringBuffer("BAD VALUE");
               }
               else
               {  Tcl_DStringAppend(&setCmd, " {", -1);
                  Tcl_DStringAppend(&setCmd, interp->result, -1);
                  aCmd = tmpStringv[tmpStringIndex++] = NXCopyStringBuffer(Tcl_DStringAppend(&setCmd, "}", -1));
               }
               Tcl_ResetResult(interp); 
   
               //NXLogError("writing out: <%s> at address %d\n", aCmd, aCmd);
               NXWriteType(stream, "*", &aCmd);
               Tcl_DStringFree(&setCmd);
            }
            if (arrayName)  {  free(arrayName); }
         }
      }
   }
   if (argv) {  free(argv); }
   
   ret = Tcl_GlobalEval(interp, "info procs"); 
   if (ret != TCL_OK)
   {  NXLogError("Yikes! problem in WWTCLInterp's write: (evaluating <info procs> yielded <%s>)\n", 
	         interp->result);
      [self showError:interp->result];
   }
   procNameList = NXCopyStringBuffer(interp->result); 
   Tcl_ResetResult(interp); 

   Tcl_SplitList(interp, procNameList, &argc, &argv);
   //NXLogError("writing out: <%d>\n", argc);
   NXWriteType(stream, "i", &argc);

   // we now check to see if these strings are going to put us over the limit
   // on our tmp string array.  If they will, realloc.
   while ((tmpStringIndex + cnt) > tmpStringc)
   {  tmpStringc *= 2;
      tmpStringv = (char **)NXZoneRealloc([self zone], tmpStringv, (sizeof(char *) * tmpStringc));
   }

   for (i = 0; i < argc; i++)
   {  Tcl_DStringInit(&procCmd);
      Tcl_DStringAppend(&procCmd, "proc ", -1);
      Tcl_DStringAppend(&procCmd, argv[i], -1);
      Tcl_DStringAppend(&procCmd, " { ", -1);

      Tcl_DStringInit(&infoCmd);
      Tcl_DStringAppend(&infoCmd, "info args ", -1);
      aCmd = Tcl_DStringAppend(&infoCmd, argv[i], -1);
      ret = Tcl_GlobalEval(interp, aCmd);
      if (ret != TCL_OK)
      {  NXLogError("Yikes! problem in WWTCLInterp's write: (evaluating <%s> yielded <%s>)\n", 
	            aCmd, interp->result);
         [self showError:interp->result];
         Tcl_ResetResult(interp); 
         aCmd = tmpStringv[tmpStringIndex++] = NXCopyStringBuffer(Tcl_DStringAppend(&procCmd, 
                                                                                    "} {echo {this proc had bad args\\n} }", -1));
      }
      else
      {  Tcl_DStringAppend(&procCmd, interp->result, -1);
         Tcl_DStringAppend(&procCmd, " } { ", -1);
         Tcl_ResetResult(interp); 
         Tcl_DStringFree(&infoCmd);

         Tcl_DStringInit(&infoCmd);
         Tcl_DStringAppend(&infoCmd, "info body ", -1);
         aCmd = Tcl_DStringAppend(&infoCmd, argv[i], -1);
         ret = Tcl_GlobalEval(interp, aCmd);
         if (ret != TCL_OK)
         {  NXLogError("Yikes! problem in WWTCLInterp's write: (evaluating <%s> yielded <%s>)\n", 
   	               aCmd, interp->result);
            [self showError:interp->result];
            aCmd = tmpStringv[tmpStringIndex++] = NXCopyStringBuffer(Tcl_DStringAppend(&procCmd, 
                                                                                    "echo {this proc had a bad body\\n} }", -1));
         }
         else
         {  Tcl_DStringAppend(&procCmd, interp->result, -1);
            Tcl_DStringFree(&infoCmd);

            aCmd = tmpStringv[tmpStringIndex++] = NXCopyStringBuffer(Tcl_DStringAppend(&procCmd, " }", -1));
         }
         Tcl_ResetResult(interp); 
      }
      //NXLogError("writing out: <%s> at address %d\n", aCmd, aCmd);
      NXWriteType(stream, "*", &aCmd);
      Tcl_DStringFree(&procCmd);

   }
   if (argv) {  free(argv); }
   if (procNameList) {  free(procNameList); }

   return self;
}

@end
