/* WWTTTimer.h
 * Written By:  Thomas Burkholder
 *
 * You may freely copy, distribute, and reuse the code in this example.
 * NeXT disclaims any warranty of any kind, expressed or  implied, as to its
 * fitness for any particular use.
 */

#import <appkit/appkit.h>

// why even declare this and clutter up the namespace?
//void handler(DPSTimedEntry tag, double now, char *data);

// An action-emitting object that also carries state.
@interface WWTTTimer:Object
{

  id    interp;
  id    startButton;
  id    stopButton;
  id    pauseButton;

        BOOL            autoStart;
        BOOL            paused;
	id			target;
	SEL			action;
	DPSTimedEntry	entry;
	double		period;
	int			priority;
	struct {
#ifdef __BIG_ENDIAN__
		unsigned int	visibleDebug:1;
		unsigned int	wrap:1;
		unsigned int	sync:1;
		unsigned int	RESERVED:5;
#else
		unsigned int	RESERVED:5;
		unsigned int	sync:1;
		unsigned int	wrap:1;
		unsigned int	visibleDebug:1;
#endif
	} bFlags;

	int			syncValue;
	float			value;
	float			startValue;
	float			wrapValue;
	float			incrementBy;

	// ivars that we don't want to archive
	unsigned		int syncTime;
	int			syncCounter;
	IMP			actionFunc;
        BOOL   awakeInIB;

  int   preControlStringSize;
  char  *preControlString;

  int   conditionalSize;
  char  *conditional;  // my "reason for living"

  int   controlStringSize;
  char  *controlString;
  char  *sprintfControlString;

  int   postControlStringSize;
  char  *postControlString;

  char  *tclCommand;
  int   tclCommandSize;
  BOOL  tclCommandDirty;
  BOOL  finiteLife;
  BOOL  conditionalLifespan;
}

// initializing and freeing the object
- init;
- free;

// action methods
- start:sender;
- stop:sender;
- pause:sender;
- resume:sender;

// methods to get/set the value of the timer
- (float)floatValue;
- setFloatValue:(float)aValue;
- (int)intValue;
- setIntValue:(int)aValue;
- (const char *)stringValue;
- setStringValue:(const char *)aValue;
- (double)doubleValue;
- setDoubleValue:(double)aValue;
- (float)incrementBy;

// methods to get/set behaviours
- target;
- setTarget:anObject;
- (SEL)action;
- setAction:(SEL)anAction;
- (DPSTimedEntry) entry;
- (double)period;
- setPeriod:(double)aPeriod;
- (int)priority;
- setPriority:(int)aPriority;
- (BOOL)visibleDebug;
- setVisibleDebug:(BOOL)yn;
- (float)startValue;
- setStartValue:(float)aValue;

- (BOOL)autoStart;
- setAutoStart:(BOOL)flag;

// Synchronization
- (unsigned int)currentTime;
- (BOOL)sync;
- setSync:(BOOL)yn;
- (int)syncValue;
- setSyncValue:(int)aValue;
- synchronize:sender;

// Wrapping
- (BOOL)wrap;
- setWrap:(BOOL)yn;
- (float)wrapValue;
- setWrapValue:(float)aValue;
- setIncrementBy:(float)aValue;

// archival methods
- read:(NXTypedStream *)stream;
- write:(NXTypedStream *)stream;

// private!! for subclasses only
- logError:(char *)str;
- stopEntry;
- startEntry;


// WavesWorld stuff
- setInterp:newInterp;
- setPreControlString:(const char *)str;
- (const char *)preControlString;
- setControlString:(const char *)str;
- (const char *)controlString;
- setPostControlString:(const char *)str;
- (const char *)postControlString;
- setTclCommand:(const char *)str;
- (const char *)tclCommand;
- setConditional:(const char *)str;
- (const char *)conditional;

- updateTclCommand;
- updateInterp;

// new stuff from wave
- (BOOL)finiteLife;
- setFiniteLife:(BOOL)n;
- (BOOL)conditionalLifespan;
- setConditionalLifespan:(BOOL)n;
- (BOOL)checkConditional;


- takePeriod:sender;
- takeStartValue:sender;
- takeWrapValue:sender;
- takeIncrementBy:sender;
- takeFiniteLife:sender;
 
@end
