/*
 * BMGranite.sl -- RenderMan compatible shader for granite.
 *
 * DESCRIPTION:
 *   Makes a granite surface.  Does basic antialiasing.
 * 
 * PARAMETERS:
 *   Ka, Kd 	   	work just like the matte shader
 *   frequency          the lowest frequency to start with
 *
 *
 * AUTHOR: written by Larry Gritz, inspired by [Upstill89]
 *         contact: gritz@seas.gwu.edu
 *
 * HISTORY:
 *      6 Aug 1991 -- first written by lg, inspired by Pixar
 *      11 Jan 1994 -- recoded by lg in correct shading language.
 *
 * last modified  11 Jan 1994 by Larry Gritz
 */



surface
BMGranite ( float Ka = 0.2, Kd = 0.8;
	    float frequency = 1; )
{
  point PP, Nf;
  float sum, period, i, fwidth, residue;

  /* Shade in shader space */
  PP = frequency * transform ("shader", P);

  /* Get the approximate filter width based on our sampling rate */
  fwidth = 0.25 * frequency * sqrt (area(P));

  /* Sum noise values until we reach the filter width */
  sum = 0;
  period = 1;
  for (i = 0;  i < 6  &&  period > fwidth;  i += 1) {
      sum += period * abs (.5 - noise (PP));
      period /= 2;
      PP *= 2;
    }

  /* Residue tells us how much of the next octave we can safely add */
  residue = clamp (2*period/fwidth-1, 0, 1);

  /* Add a portion of the next octave of noise */
  sum += residue * noise (PP) * period;

  /* And add the remainder corresponding to the octaves we didn't get to */
  sum += (2-residue)*period/2;

  /* Use the matte illumination model */
  Oi = Os;
  Ci = Os * Cs * sum * (Ka * ambient() + Kd * diffuse(faceforward(normalize(N),I)));
}
